SystemOrganization addCategory: #'Math-DHB-Numerical'!
SystemOrganization addCategory: 'Math-DHB-Numerical-Tests'!

!Number class methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number between 0 and the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^ DhbMitchellMooreGenerator new floatValue! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
beta: aNumber
		"Computes the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self logBeta: aNumber) exp! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equalsTo: aNumber
		"compare to Float>>closeTo:
		
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	^self relativelyEqualsTo: aNumber upTo: DhbFloatingPointMachine new defaultNumericalPrecision! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorFunction
		"Answer the error function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ DhbErfApproximation new value: self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
		"Compute the Gamma function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ self > 1
		ifTrue: [ ^ DhbLanczosFormula new gamma: self]
		ifFalse:[ self < 0
						ifTrue: [ Float pi / ( ( Float pi * self) sin * ( 1 - self) gamma)]
						ifFalse:[ ( DhbLanczosFormula new gamma: ( self + 1)) / self]
					]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logBeta: aNumber
		"Computes the logarithm of the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self logGamma + aNumber logGamma - ( self + aNumber) logGamma! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logGamma
		"Computes the log of the Gamma function (for positive numbers only)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self > 1
		ifTrue: [ DhbLanczosFormula new logGamma: self]
		ifFalse:[ self > 0
						ifTrue: [ ( DhbLanczosFormula new logGamma: ( self + 1) ) - self ln ]
						ifFalse: [ ^self error: 'Argument for the log gamma function must be positive']
					]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'WernerKassens 5/2/2014 09:10'!
productWithMatrix: aMatrix
^aMatrix class rows: (aMatrix rowsCollect: [:r| self productWithVector: r])! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number distributed between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^self class random * self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
relativelyEqualsTo: aNumber upTo: aSmallNumber
		"compare to Float>>closeTo:
		generally called from Number>>equalsTo:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| norm |
	norm := self abs max: aNumber abs.
	^norm <= DhbFloatingPointMachine new defaultNumericalPrecision
		or: [ (self - aNumber) abs < ( aSmallNumber * norm)]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: self! !

!Integer methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
	self > 0
		ifFalse: [^ self error: 'Attempt to compute the Gamma function of a non-positive integer'].
	^ (self - 1) factorial.! !

!Integer methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random integer between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^ DhbMitchellMooreGenerator new integerValue: self! !

!Collection methodsFor: '*Math-DHB-Numerical' stamp: 'WernerKassens 4/15/2014 09:56'!
asDhbVector
	"Convert the receiver to a vector. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector! !

ArithmeticError subclass: #SingularMatrixError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!SingularMatrixError commentStamp: 'WernerKassens 4/5/2014 11:53' prior: 0!
some calculations dont work with singular matrices and result eg with errors like 'receiver of last is nil', deep down in the calculation, which is quite ununderstandable. otoh these errors can occasionally be treated by using another calculation, hence i introduced this error.
!

!SingularMatrixError methodsFor: 'accessing' stamp: 'WernerKassens 4/5/2014 11:35'!
messageText
	"Overwritten to initialize the message text to a standard text if it has not yet been set"
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!SingularMatrixError methodsFor: 'printing' stamp: 'WernerKassens 4/6/2014 11:16'!
standardMessageText
	^ String streamContents: [ :stream |
		stream << (self signalerContext  asString findTokens: '>')last.
		stream << ' encountered a singular matrix']! !

Array variableSubclass: #DhbVector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
* aNumberOrMatrixOrVector
		"Answers the product of the receiver with the argument.
		 The argument can be a number, matrix or vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aNumberOrMatrixOrVector productWithVector: self! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
+ aVector
		"Answers the sum of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a + b).
		].
	^answer! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
- aVector
		"Answers the difference of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer n |
	answer := self class new: self size.
	n := 0.
	self with: aVector do:
		[ :a :b | 
		  n := n + 1. 
		  answer at: n put: ( a - b).
		].
	^answer! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVectorOrAnArray
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) + ( aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulateNegated: aVectorOrAnArray
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) - ( aVectorOrAnArray at: n))].! !

!DhbVector methodsFor: 'information' stamp: 'WernerKassens 4/15/2014 09:56'!
asDhbVector
		"Answer self since the receiver is a vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negate
		"Inverse the sign of all components of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	1 to: self size do: [ :n | self at: n put: (self at: n) negated].! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negated
	^ self * -1! !

!DhbVector methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
norm
		"Answer the norm of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^(self * self) sqrt! !

!DhbVector methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalized
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30-Dec-99 "
	^(1 / self norm) * self! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithMatrix: aMatrix
		"Answers the product of aMatrix with the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix rowsCollect: [ :each | each * self]! !

!DhbVector methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers the scalar product of aVector with the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^self inject: 0
			into: [ :sum :each | n := n + 1. (aVector at: n) * each + sum]! !

!DhbVector methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 08:38'!
scalarProduct: aVector
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/5/99 "
	| product n |
	n := 0.
	product := self collect: [ :each | n := n + 1. (aVector at: n) * each].
	n := product size.
	[ n > 1]
		whileTrue:[ | i j|
					i := 1.
					j := n.
					[ i < j]
						whileTrue: [ product at: i put: ( product at: i) + ( product at: j).
									 j := j - 1.
									 i := i + 1.
								   ].
					n := i min: j.
				  ].
	^product at: 1! !

!DhbVector methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
scaleBy: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	1 to: self size do: [ :n | self at: n put: ( ( self at: n) * aNumber)].! !

!DhbVector methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 10:16'!
tensorProduct: aVector
		"Answers the tensor product of the receiver with aVector."
	^self == aVector "dont use #="
			ifTrue: [DhbSymmetricMatrix new: self size function:[:x :y|x*y]] 
			ifFalse:[DhbMatrix rows: ( self collect: [ :a | aVector collect: [ :b | a * b]])]
! !

TestCase subclass: #DhbExponentialDistributionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Test-DHB-Numerical'!

!DhbExponentialDistributionTest commentStamp: '<historical>' prior: 0!
A DhbExponentialDistributionTest is a test class for testing the behavior of DhbExponentialDistribution!

!DhbExponentialDistributionTest methodsFor: 'tests' stamp: 'SergeStinckwich 3/8/2015 21:30'!
testSampleMeanConvergesToDistributionMean
	"testing a random sample seems suspect. We use a 5% interval here"
	| eg arr |
	eg := DhbExponentialDistribution scale: 10.
	arr := Array new: 10000.
	(1 to: 10000) do: [:index | arr at: index put: eg random].
	self should: [arr average between: (eg average * 0.95) and: (eg average * 1.05) ].! !

Object subclass: #DhbAbstractDataServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbAbstractDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
close
		"Close the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

DhbAbstractDataServer subclass: #DhbMemoryBasedDataServer
	instanceVariableNames: 'data position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data size < position! !

!DhbMemoryBasedDataServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
data: anOrderedCollection
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	data := anOrderedCollection.
	self reset.! !

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dimension
		"Answers the dimension of the vectors catered by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data first size! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| answer |
	answer := data at: position.
	position := position + 1.
	^answer! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self reset! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	position := 1.! !

Object subclass: #DhbChromosomeManager
	instanceVariableNames: 'population populationSize rateOfMutation rateOfCrossover'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbChromosomeManager class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger mutation: aNumber1 crossover: aNumber2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self new populationSize: anInteger; rateOfMutation: aNumber1; rateOfCrossover: aNumber2; yourself! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
clone: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^aChromosome copy! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isFullyPopulated
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population size >= populationSize! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
mutate: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
population
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
populationSize: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	populationSize := anInteger.! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
process: aChromosome1 and: aChromosome2 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	| roll |
	roll := Number random.
	roll < rateOfCrossover 
		ifTrue: [population addAll: (self crossover: aChromosome1 and: aChromosome2)]
		ifFalse: 
			[roll < (rateOfCrossover + rateOfMutation) 
				ifTrue: 
					[population
						add: (self mutate: aChromosome1);
						add: (self mutate: aChromosome2)]
				ifFalse: 
					[population
						add: (self clone: aChromosome1);
						add: (self clone: aChromosome2)]]! !

!DhbChromosomeManager methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomChromosome
	self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomizePopulation
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	self reset.
	[self isFullyPopulated] 
		whileFalse: [population add: self randomChromosome]! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rateOfCrossover: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of cross-over'].
	rateOfCrossover := aNumber! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rateOfMutation: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of mutation'].
	rateOfMutation := aNumber! !

!DhbChromosomeManager methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	population := OrderedCollection new: populationSize.! !

DhbChromosomeManager subclass: #DhbVectorChromosomeManager
	instanceVariableNames: 'origin range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index new1 new2|
	index := ( aChromosome1 size - 1) random + 2.
	new1 := self clone: aChromosome1.
	new1 replaceFrom: index to: new1 size with: aChromosome2 startingAt: index.
	new2 := self clone: aChromosome2.
	new2 replaceFrom: index to: new2 size with: aChromosome1 startingAt: index.
	^Array with: new1 with: new2! !

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
mutate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index |
	index := aVector size random + 1.
	^( aVector copy)
			at: index put: ( self randomComponent: index);
			yourself! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	origin := aVector.! !

!DhbVectorChromosomeManager methodsFor: 'creation' stamp: 'WernerKassens 4/15/2014 09:56'!
randomChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( ( 1 to: origin size) collect: [ :n | self randomComponent: n]) asDhbVector! !

!DhbVectorChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomComponent: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( range at: anInteger) asFloat random + ( origin at: anInteger)! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	range := aVector.! !

Object subclass: #DhbCluster
	instanceVariableNames: 'accumulator previousSampleSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCluster class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^super new initialize! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changes
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	^(self sampleSize - previousSampleSize) abs! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := 0.
	^self! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isInsignificantIn: aClusterFinder
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/17/00 "
	^self sampleSize <= aClusterFinder minimumClusterSize! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := self sampleSize.
	self collectAccumulatorResults.
	accumulator reset! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sampleSize
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

DhbCluster subclass: #DhbCovarianceCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
centerOn: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/16/00"

	accumulator := aVector isNil
				ifTrue: [nil]
				ifFalse: [DhbMahalanobisCenter onVector: aVector]! !

!DhbCovarianceCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator computeParameters.! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator distanceTo: aVector! !

!DhbCovarianceCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator isNil! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator printOn: aStream.! !

DhbCluster subclass: #DhbEuclideanCluster
	instanceVariableNames: 'center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'SergeStinckwich 1/25/2013 13:57'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
		"minimal change so that it works in pharo 1.2.1" 
	center := aVector.
	accumulator := DhbVectorAccumulator new: (aVector ifNil:[0]ifNotNil:[aVector size]).! !

!DhbEuclideanCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.! !

!DhbEuclideanCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^( aVector - center) norm! !

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^center isNil! !

!DhbEuclideanCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

Object subclass: #DhbCongruentialRandomNumberGenerator
	instanceVariableNames: 'constant modulus multiplicator seed'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
constant: aNumber1 multiplicator: aNumber2 modulus: aNumber3 
	"Create a new instance of the receiver with given constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new 
		initialize: aNumber1
		multiplicator: aNumber2
		modulus: aNumber3! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new initialize.
					 UniqueInstance setSeed: 1.
				   ].
	^UniqueInstance! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
seed: aNumber
		"Create a new instance of the receiver with given seed
		  using D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( super new) initialize; setSeed: aNumber; yourself! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
floatValue
		"Answer the next pseudo-random value between 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self value asFloat / modulus! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Initializes the constants of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self initialize: 2718281829.0 multiplicator: 3141592653.0 modulus: 4294967296.0.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 multiplicator: aNumber2 modulus: aNumber3
		"Private - Initializes the constants needed by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	constant := aNumber1.
	modulus := aNumber2.
	multiplicator := aNumber3.
	self setSeed: 1.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerValue: anInteger
		"Answer a random integer between 0 and the anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( self value  \\ ( anInteger * 1000)) // 1000! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setSeed: aNumber
		"Set the seed of the receiver to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := aNumber.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"Answer the next pseudo-random value.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := ( seed * multiplicator + constant) \\ modulus.
	^seed! !

Object subclass: #DhbDecimalFloatingNumber
	instanceVariableNames: 'mantissa exponent'
	classVariableNames: 'Digits'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbDecimalFloatingNumber commentStamp: '<historical>' prior: 0!
A DhbDecimalFloatingNumber is a demonstration of floating point arithmetic and rounding errors. The implementation uses decimal digits to allow easier human reading and understanding, and hand-worked examples. Only basic arithmetic operations are specified (+, -,*, sqrt). 

The message #value answers a fraction which is the exact representation of the stored number. 

The message #normalize uses truncation to fit the best precision possible. It is at this step that roundoff errors occur. #normalize is called when these numbers are created.

This class is detailed in appendix A of Didier Besset's book.!

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^15! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDigits: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := anInteger.! !

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
digits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits isNil
		ifTrue: [ Digits := self defaultDigits].
	^Digits! !

!DhbDecimalFloatingNumber class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self new normalize: aNumber! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := nil.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
* aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value * aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
+ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value + aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
- aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value - aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
/ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value / aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalize: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	exponent := (self class digits - (aNumber log: 10)) floor.
	mantissa := ( aNumber * ( 10 raisedToInteger: exponent)) truncated.
	^self! !

!DhbDecimalFloatingNumber methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	mantissa printOn: aStream.
	aStream nextPutAll: 'xE'.
	exponent negated printOn: aStream.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sqrt
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new:  self value sqrt! !

!DhbDecimalFloatingNumber methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^mantissa / ( 10 raisedToInteger: exponent)! !

Object subclass: #DhbErfApproximation
	instanceVariableNames: 'constant series norm'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbErfApproximation commentStamp: '<historical>' prior: 0!
A DhbErfApproximation is a five term approximation to erf(x). This is a singleton class encapsulating the approximation. 

Use with 

DhbErfApproximation new value: aNumber

to receive erf(x). The result of the error function lies between zero and one.

An extension to Number exists, so the simpler use is

aNumber errorFunction 

which produces the same result.

If you need the error function as a function, you will need to enclose it in a block as:

| errorFunction |
errorFunction := [:x | x errorFunction].

Instance variables constant and series are part of the approximation formula. norm is a scale factor to make erf(infinity) = 1.

The error function is the Cumulative Distribution of the standard normal distribution. Thus, erf(x) represents the probability of a random variable with standard normal distribution being less than x. The approximation used is credited to Abramowitz and Stegun's Handbook of Mathematical Functions. The error function is detailed in Chapter 7. The implementation is detailed in Besset's book Section 2.3

Additional resources available from NIST Digital Library of Mathematics at:
http://dlmf.nist.gov/7!

!DhbErfApproximation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance! !

!DhbErfApproximation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"ensure a new UniqueInstance is built when used next"
	UniqueInstance := nil.! !

!DhbErfApproximation methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Initialize constants needed to evaluate the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	constant := 0.2316419.
	norm := 1 / ( Float pi * 2) sqrt.
	series := DhbPolynomial coefficients: #( 0.31938153 -0.356563782 1.781477937 -1.821255978 1.330274429).! !

!DhbErfApproximation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normal: aNumber
		"Computes the value of the Normal distribution for aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "

	^[ ( aNumber squared * -0.5) exp * norm]
			on: Error do: [ :signal | signal return: 0]! !

!DhbErfApproximation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answer erf( aNumber) using an approximation from Abramovitz and Stegun, Handbook of Mathematical Functions.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/1/99 "
	| t |
	aNumber = 0
		ifTrue: [ ^0.5].
	aNumber > 0
		ifTrue: [ ^1- ( self value: aNumber negated)].
	aNumber < -20
		ifTrue: [ ^0].
	t := 1 / (1 - (constant * aNumber)).
	^( series value: t) * t * (self normal: aNumber)! !

Object subclass: #DhbFloatingPointMachine
	instanceVariableNames: 'defaultNumericalPrecision radix machinePrecision negativeMachinePrecision smallestNumber largestNumber smallNumber largestExponentArgument'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFloatingPointMachine commentStamp: '<historical>' prior: 0!
A DhbFloatingPointMachine represents the numerical precision of this system.

Instance Variables

	defaultNumericalPrecision:		The relative 
numerical precision that can be expected for a general numerical computation. One should consider to numbers a and b equal if the relative difference between them is less than the default machine precision.

	largestExponentArgument:		natural logarithm of largest number

	largestNumber:		The largest positive number that can be represented in the machine

	machinePrecision:		r^{-(n+1)}, with the largest n such that (1 + r^-n) - 1 !!= 0

	negativeMachinePrecision:		r^{-(n+1)}, with the largest n such that (1 - r^-n) - 1 !!= 0

	radix:		The radix of the floating point representation. This is often 2.

	smallNumber:		A number that can be added to some value without noticeably changing the result of the computation

	smallestNumber:		The smallest positive number different from 0.


largestExponentArgument
	- xxxxx

This class is detailed in Object Oriented Implementation of Numerical Methods, Section 1.4.1 and 1.4.2!

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'wernerkassens 10/29/2014 15:16'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	UniqueInstance ifNil: [ UniqueInstance := super new].
	^UniqueInstance! !

!DhbFloatingPointMachine class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	UniqueInstance := nil.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeLargestNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/6/99 "
	| one floatingRadix fullMantissaNumber |

	one := 1.0.
	floatingRadix := self radix asFloat.
	fullMantissaNumber := one - ( floatingRadix * self negativeMachinePrecision).
	largestNumber := fullMantissaNumber.
	[ [ fullMantissaNumber := fullMantissaNumber * floatingRadix.
	    fullMantissaNumber isInfinite ifTrue: [^nil].
	    largestNumber := fullMantissaNumber.
		true] whileTrue: [ ].
		] on: Error do: [ :signal | signal return: nil].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeMachinePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	inverseRadix := one / self radix asFloat.
	machinePrecision := one.
	[ tmp := one + machinePrecision.
	  tmp - one = zero]
		whileFalse:[  machinePrecision := machinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNegativeMachinePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero floatingRadix inverseRadix tmp |
	one := 1.0.
	zero := 0.0.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	negativeMachinePrecision := one.
	[ tmp := one - negativeMachinePrecision.
	  tmp - one = zero]
		whileFalse:[ negativeMachinePrecision := negativeMachinePrecision * inverseRadix].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeRadix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| one zero a b tmp1 tmp2|
	one := 1.0.
	zero := 0.0.
	a := one.
	[ a := a + a.
	  tmp1 := a + one.
	  tmp2 := tmp1 - a.
	  tmp2 - one = zero] whileTrue:[].
	b := one.
	[ b := b + b.
	  tmp1 := a + b.
	  radix := ( tmp1 - a) truncated.
	  radix = 0 ] whileTrue: [].! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'wernerkassens 10/29/2014 14:39'!
computeSmallestNumber
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/6/99"

	| one floatingRadix inverseRadix fullMantissaNumber |
	one := 1 asFloat.
	floatingRadix := self radix asFloat.
	inverseRadix := one / floatingRadix.
	fullMantissaNumber := one - (floatingRadix * self negativeMachinePrecision).
	smallestNumber := fullMantissaNumber.
	[[fullMantissaNumber := fullMantissaNumber * inverseRadix.
	fullMantissaNumber = 0.0 ifTrue: [Error signal ].
	smallestNumber := fullMantissaNumber.
	true]
		whileTrue: []]
		on: Error do: [:signal | signal return: nil]! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultNumericalPrecision
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	defaultNumericalPrecision isNil
		ifTrue: [ defaultNumericalPrecision := self machinePrecision sqrt].
	^defaultNumericalPrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestExponentArgument
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "
	largestExponentArgument isNil
		ifTrue: [ largestExponentArgument := self largestNumber ln].
	^largestExponentArgument! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	largestNumber isNil
		ifTrue: [ self computeLargestNumber].
	^largestNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
machinePrecision
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	machinePrecision isNil
		ifTrue: [ self computeMachinePrecision].
	^machinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negativeMachinePrecision
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	negativeMachinePrecision isNil
		ifTrue: [ self computeNegativeMachinePrecision].
	^negativeMachinePrecision! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
radix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	radix isNil
		ifTrue: [ self computeRadix].
	^radix! !

!DhbFloatingPointMachine methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
showParameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/6/99 "
		"added 'Transcript flush' to make sure that the transcript window gets updated"
	Transcript cr; cr;
			nextPutAll: 'Floating-point machine parameters'; cr;
			nextPutAll: '---------------------------------';cr;
			nextPutAll: 'Radix: '.
	self radix printOn: Transcript.
	Transcript cr; nextPutAll: 'Machine precision: '.
	self machinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Negative machine precision: '.
	self negativeMachinePrecision printOn: Transcript.
	Transcript cr; nextPutAll: 'Smallest number: '.
	self smallestNumber printOn: Transcript.
	Transcript cr; nextPutAll: 'Largest number: '.
	self largestNumber printOn: Transcript.
	Transcript flush.! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
smallNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	smallNumber isNil
		ifTrue: [ smallNumber := self smallestNumber sqrt].
	^smallNumber! !

!DhbFloatingPointMachine methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
smallestNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	smallestNumber isNil
		ifTrue: [ self computeSmallestNumber].
	^smallestNumber! !

Object subclass: #DhbHistogram
	instanceVariableNames: 'minimum binWidth overflow underflow moments contents freeExtent cacheSize desiredNumberOfBins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultCacheSize
		"Private - Answer the default cache size.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^100! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultNumberOfBins
		"Private - Defines the default number of bins for instances of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^50! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerScales
	"Private - Scales for strict integers"
	^#( 2 4 5 8 10)! !

!DhbHistogram class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a standard new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
scales
	"Private - Scales for any number"
	^#( 1.25 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
semiIntegerScales
	"Private - Scales for large integers"
	^#( 2 2.5 4 5 7.5 8 10)! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber
		"Accumulate aNumber into the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| bin |
	"(contents includes: nil) ifTrue: [self halt]."
	self isCached
		ifTrue: [ ^self accumulateInCache: aNumber].

	bin := self binIndex: aNumber.
	( bin between: 1 and: contents size)
		ifTrue: [ contents at: bin put: ( contents at: bin) + 1.
					 moments accumulate: aNumber.
				   ]
		ifFalse:[ self processOverflows: bin for: aNumber].
! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulateInCache: aNumber
		"Private - Accumulate aNumber inside a cache
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	contents add: aNumber.
	contents size > cacheSize
		ifTrue: [ self flushCache].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
adjustDimensionUpTo: aNumber
		"Private - Compute an adequate bin width and adjust the minimum and number of bins accordingly.
		 aNumber is the maximum value to accumulate. The minimum value has already been assigned.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	| maximum |
	binWidth := self roundToScale: ( aNumber - minimum) / desiredNumberOfBins.
	minimum := ( minimum / binWidth) floor * binWidth.
	maximum := ( aNumber / binWidth) ceiling * binWidth.
	contents := Array new: ( ( maximum - minimum) / binWidth) ceiling.
	contents atAllPut: 0.! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:44'!
average
		"Answer the average of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments average! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
binIndex: aNumber
		"Answers the index of the bin corresponding to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^( ( aNumber - minimum) / binWidth) floor + 1! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/6/2014 13:14'!
binWidth
		"Answer the bin width for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache]. 
	^binWidth! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chi2Against: aScaledDistribution
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/30/00 "
	| chi2 |
	chi2 := 0.
	self pointsAndErrorsDo:
		[ :each | chi2 := ( each chi2Contribution: aScaledDistribution) + chi2].
	^chi2! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chi2ConfidenceLevelAgainst: aScaledDistribution
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/30/00 "
	^( DhbChiSquareDistribution degreeOfFreedom: ( contents size - aScaledDistribution parameters size))
			confidenceLevel: ( self chi2Against: aScaledDistribution)! !

!DhbHistogram methodsFor: 'information' stamp: 'wernerkassens 10/29/2014 14:59'!
collectIntegralPoints: aBlock
		"Collects the points needed to display the receiver as an integral.
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	| answer bin integral norm x |
	self isCached
		ifTrue: [ self flushCache].
	answer := OrderedCollection new: ( contents size * 2 + 1).
	bin := self minimum.
	answer add: ( aBlock value: bin @ 0).
	integral := self underflow.
	norm := self totalCount.
	contents do:
		[ :each |
		  integral := integral + each.
		  x := integral / norm.
		  answer add: ( aBlock value: bin @ x).
		  bin := bin + binWidth.
		  answer add: ( aBlock value: bin @ x).
		].
	answer add: ( aBlock value: bin @ 0).
	^answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'wernerkassens 10/29/2014 14:58'!
collectPoints: aBlock
		"Collects the points needed to display the receiver.
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| answer bin |
	self isCached
		ifTrue: [ self flushCache].
	answer := OrderedCollection new: ( contents size * 2 + 1).
	bin := self minimum.
	answer add: ( aBlock value: bin @ 0).
	contents do:
		[ :each |
		  answer add: ( aBlock value: bin @ each).
		  bin := bin + binWidth.
		  answer add: ( aBlock value: bin @ each).
		].
	answer add: ( aBlock value: bin @ 0).
	^answer asArray! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:45'!
count
		"Answer the count of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments count! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 13:05'!
countAt: aNumber
		"Answer the count in the bin corresponding to aNumber or 0 if outside the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n |
	self isCached 				
		ifTrue: [ self flushCache].
	n := self binIndex: aNumber.
	^( n between: 1 and: contents size)
			ifTrue: [ contents at: n]
			ifFalse:[ 0]! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
countOverflows: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	anInteger > 0
		ifTrue: [ overflow := overflow + 1]
		ifFalse:[ underflow := underflow + 1].! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 15:05'!
countsBetween: aNumber1 and: aNumber2
		"Computes the events located between aNumber1 and aNumber2.
		 NOTE: This method assumes the two numbers are within the limits
			   of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
		"there was a little bug that just seems to be one of the authors typical slips of the pen"
	| n1 n2 answer |
	self isCached 				
		ifTrue: [ self flushCache].
	n1 := self binIndex: aNumber1.
	n2 := self binIndex: aNumber2.
	answer := ( contents at: n1) * ( ( binWidth * (n1-1) + minimum) - aNumber1) / binWidth.
	n2 > contents size
		ifTrue: [ n2 := contents size + 1]
		ifFalse:[ answer := answer + ( ( contents at: n2) * ( aNumber2 - ( binWidth *  (n2-1) + self minimum)) / binWidth)].
	( n1 ) to: ( n2 - 1) do: [ :n | answer := answer + ( contents at: n)].
	^answer ! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/6/2014 10:20'!
countsUpTo: aNumber
		"Computes the events located up to aNumber.
		 NOTE: This method assumes aNumber is within the limits
					 of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	| n answer |
	self isCached 				
		ifTrue: [ self flushCache].
	n := self binIndex: aNumber.
	n > contents size
		ifTrue: [ ^self count].
	answer := ( contents at: n) * ( aNumber - ( binWidth * ( n - 1) + minimum)) / binWidth.
	1 to: ( n - 1) do: [ :m | answer := answer + ( contents at: m)].
	^answer + underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:46'!
errorOnAverage
		"Answer the error on the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments errorOnAverage! !

!DhbHistogram methodsFor: 'testing' stamp: 'WernerKassens 4/5/2014 13:02'!
fConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^self moments fConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
flushCache
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| maximum values |
	minimum isNil ifTrue: [minimum := contents isEmpty
					ifTrue: [0]
					ifFalse: [contents first]].
	maximum := minimum.
	contents do: [:each | each < minimum
			ifTrue: [minimum := each]
			ifFalse: [each > maximum ifTrue: [maximum := each]]].
	maximum = minimum ifTrue: [maximum := minimum + desiredNumberOfBins].
	values := contents.
	self adjustDimensionUpTo: maximum.
	values do: [:each | self accumulate: each]! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
freeExtent: aBoolean
		"Defines the range of the receiver to be freely adjustable.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	( underflow = 0 and: [ overflow = 0])
		ifFalse: [ self error: 'Histogram extent cannot be redefined'].
	freeExtent := aBoolean.! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
growContents: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	anInteger > 0
		ifTrue: [ self growPositiveContents: anInteger]
		ifFalse:[ self growNegativeContents: anInteger].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
growNegativeContents: anInteger 
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| n newSize newContents |
	n := 1 - anInteger.
	newSize := contents size + n.
	newContents := Array new: newSize.
	newContents at: 1 put: 1.
	2 to: n do: [:i | newContents at: i put: 0].
	newContents
		replaceFrom: n + 1
		to: newSize
		with: contents
		startingAt: 1.
	contents := newContents.
	minimum := anInteger - 1 * binWidth + minimum! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
growPositiveContents: anInteger 
	"Private - 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 15/2/99"

	| n newContents |
	n := contents size.
	newContents := Array new: anInteger.
	newContents
		replaceFrom: 1
		to: n
		with: contents
		startingAt: 1.
	n + 1 to: anInteger - 1 do: [:i | newContents at: i put: 0].
	newContents at: anInteger put: 1.
	contents := newContents! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - initializes the receiver with standard settings.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	freeExtent := false.
	cacheSize := self class defaultCacheSize.
	desiredNumberOfBins := self class defaultNumberOfBins.
	contents := OrderedCollection new: cacheSize.
	moments := DhbFixedStatisticalMoments new.
	overflow := 0.
	underflow := 0.
	^self! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 13:32'!
inverseDistributionValue: aNumber
		"Private - Compute the value which corresponds to a integrated count of aNumber.
		 NOTE: aNumber is assumed to be between 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| count x integral |
	count := self count * aNumber.
	x := minimum.
	integral := 0.
	contents do:
		[ :each | | delta |
		  delta := count - integral.
		  each > delta
			ifTrue: [ ^binWidth * delta / each + x].
		  integral := integral + each.
		  x := binWidth + x.
		].
	^self maximum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isCached
		"Private - Answer true if the content of the receiver is cached.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^binWidth isNil! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isEmpty
		"Always false. 
		 Needed to use polymorphic behavior when plotting the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^false! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:47'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments kurtosis! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 13:07'!
lowBinLimitAt: anInteger
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	self isCached 				
		ifTrue: [ self flushCache].
	^( anInteger - 1) * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents size * binWidth + minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximumCount
		"Answer the maximum count of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^contents inject: ( contents isEmpty ifTrue: [ 1] ifFalse:[ contents at: 1])
					into: [ :max :each | max max: each]! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimum
		"Answer the minimum for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifTrue: [ self flushCache].
	^minimum! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:44'!
moments
self isCached
	ifTrue: [ self flushCache].
^moments! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
overflow
		"Answer the overflow of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^overflow! !

!DhbHistogram methodsFor: 'iterators' stamp: 'WernerKassens 4/5/2014 19:43'!
pointsAndErrorsDo: aBlock
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| x |
	x := self minimum - ( binWidth / 2).
	contents do:
		[ :each |
		  x := x + binWidth.
		  aBlock value: ( DhbWeightedPoint point: x count: each).
		].! !

!DhbHistogram methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
processOverflows: anInteger for: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	freeExtent
		ifTrue: [ self growContents: anInteger.
					 moments accumulate: aNumber
				   ]
		ifFalse:[ self countOverflows: anInteger].! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
roundToScale: aNumber
	"Private - Adjust aNumber of the lowest upper scale"
	| orderOfMagnitude norm scales rValue |
	orderOfMagnitude := ( aNumber log: 10) floor.
	scales := self class scales.
	aNumber isInteger
		ifTrue: [ orderOfMagnitude < 1
						ifTrue: [ orderOfMagnitude := 1].
					  orderOfMagnitude = 1
						ifTrue: [ scales := self class integerScales].
					  orderOfMagnitude = 2
						ifTrue: [ scales := self class semiIntegerScales].
				    ].
	norm := 10 raisedToInteger: orderOfMagnitude.
	rValue := aNumber / norm.
	^( scales detect: [ :each | rValue <= each]) * norm! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setDesiredNumberOfBins: anInteger
		"Defines the desired number of bins. It may be adjusted to a few units later on.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	anInteger > 0
		ifFalse:[ self error: 'Desired number of bins must be positive'].
	desiredNumberOfBins := anInteger.! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setRangeFrom: aNumber1 to: aNumber2 bins: anInteger
		"Defines the range of the receiver by specifying the minimum, maximum and number of bins.
		 Values are adjusted to correspond to a reasonable value for the bin width and the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined'].
	minimum := aNumber1.
	self setDesiredNumberOfBins: anInteger;
		   adjustDimensionUpTo: aNumber2.! !

!DhbHistogram methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setWidth: aNumber1 from: aNumber2 bins: anInteger
		"Defines the range of the receiver by specifying the minimum, bin width and number of bins.
		 Values are adjusted to correspond to a reasonable value for the bin width and the limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	self isCached
		ifFalse: [ self error: 'Histogram limits cannot be redefined'].
	minimum := aNumber2.
	self setDesiredNumberOfBins: anInteger;
		   adjustDimensionUpTo: ( aNumber1 * anInteger + aNumber2).! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:49'!
skewness
		"Answer the skewness of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments skewness! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:49'!
standardDeviation
		"Answer the standardDeviation of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments standardDeviation! !

!DhbHistogram methodsFor: 'testing' stamp: 'WernerKassens 4/5/2014 13:03'!
tConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^self moments tConfidenceLevel: aStatisticalMomentsOrHistogram! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:57'!
totalCount
		"Answer the count of the recevier, inclusing underflow and overflow
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments count + underflow + overflow! !

!DhbHistogram methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
underflow
		"Answer the underflow of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^underflow! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:49'!
unnormalizedVariance
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^self moments unnormalizedVariance! !

!DhbHistogram methodsFor: 'information' stamp: 'WernerKassens 4/5/2014 12:50'!
variance
		"Answer the variance of the recevier
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^self moments variance! !

Object subclass: #DhbIncompleteBetaFunction
	instanceVariableNames: 'alpha1 alpha2 fraction inverseFraction logNorm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	fraction isNil 
		ifTrue: 
			[fraction := DhbIncompleteBetaFractionTermServer new.
			fraction setParameter: alpha1 second: alpha2].
	fraction setArgument: aNumber.
	^(DhbContinuedFraction server: fraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateInverseFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	inverseFraction isNil 
		ifTrue: 
			[inverseFraction := DhbIncompleteBetaFractionTermServer new.
			inverseFraction setParameter: alpha2 second: alpha1].
	inverseFraction setArgument: (1 - aNumber).
	^(DhbContinuedFraction server: inverseFraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteBetaFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	alpha1 := aNumber1.
	alpha2 := aNumber2.
	logNorm := ( alpha1 + alpha2) logGamma - alpha1 logGamma - alpha2 logGamma.
	^self! !

!DhbIncompleteBetaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the receiver for argument aNumber.
		 Note: aNumber must be between 0 and 1 (otherwise an exception will occur)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	| norm |
	aNumber = 0
		ifTrue: [ ^0].
	aNumber = 1
		ifTrue: [ ^1].
	norm :=  ( aNumber ln * alpha1 + ( ( 1 - aNumber) ln * alpha2) + logNorm) exp.
	^( alpha1 + alpha2 + 2) * aNumber < ( alpha1 + 1)
		ifTrue: [ norm / ( ( self evaluateFraction: aNumber) * alpha1)]
		ifFalse:[ 1 - ( norm / ( ( self evaluateInverseFraction: aNumber) * alpha2))]! !

Object subclass: #DhbIncompleteGammaFunction
	instanceVariableNames: 'alpha alphaLogGamma series fraction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber
		"Defines a new instance of the receiver with paramater aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^super new initialize: aNumber! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateFraction: aNumber 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	fraction isNil 
		ifTrue: 
			[fraction := DhbIncompleteGammaFractionTermServer new.
			fraction setParameter: alpha].
	fraction setArgument: aNumber.
	^(DhbContinuedFraction server: fraction)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateSeries: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	series isNil
		ifTrue: [ series := DhbIncompleteGammaSeriesTermServer new.
				  series setParameter: alpha.
				].
	series setArgument: aNumber.
	^(DhbInfiniteSeries server: series)
		desiredPrecision: DhbFloatingPointMachine new defaultNumericalPrecision;
		evaluate! !

!DhbIncompleteGammaFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber
		"Private - Defines the parameter alpha of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	alpha := aNumber asFloat.
	alphaLogGamma := alpha logGamma.
	^self! !

!DhbIncompleteGammaFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the receiver for argument aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	| x norm |
	aNumber = 0
		ifTrue: [ ^0].
	x := aNumber asFloat.
	norm := [ ( x ln * alpha - x - alphaLogGamma) exp] on: Error do: [ :signal | signal return: nil].
	norm isNil
		ifTrue: [ ^1].
	^x - 1 < alpha
		ifTrue: [ ( self evaluateSeries: x) * norm]
		ifFalse:[ 1 - ( norm / ( self evaluateFraction: x))]! !

Object subclass: #DhbIterativeProcess
	instanceVariableNames: 'precision desiredPrecision maximumIterations result iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIterativeProcess commentStamp: '<historical>' prior: 0!
A DhbIterativeProcess class is an abstract base class for processes which follow an iterative pattern. 

Subclasses of IterativeProcess will redefine
	initializeIterations
	evaluateIteration
	finalizeIterations

The instance variable result is used to store the most recent/best result, and is accessible through the result selector.

The maximumIterations: method allows control of the amount of work this method is allowed to do. Each evaluation of the iteration increments the instance variable iterations. When this number exceeds maximumIterations, the evaluate method will stop the process, and answer result.!

DhbIterativeProcess subclass: #DhbClusterFinder
	instanceVariableNames: 'dataServer dataSetSize minimumRelativeClusterSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbClusterFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger server: aClusterDataServer type: aClusterClass
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^super new initialize: anInteger server: aClusterDataServer type: aClusterClass! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"Private - Accumulate aVector into the nearest cluster.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	( result at: ( self indexOfNearestCluster: aVector)) accumulate: aVector.! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
clusters: aCollectionOfClusters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	result := aCollectionOfClusters.! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectChangesAndResetClusters
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	| hasEmptyClusters changes |
	changes := 0.
	hasEmptyClusters := false.
	result do: 
			[:each | 
			changes := each changes + changes.
			( each isInsignificantIn: self)
				ifTrue: 
					[each centerOn: nil.
					hasEmptyClusters := true]
				ifFalse: [each reset].
			].
	hasEmptyClusters 
		ifTrue: [result := result reject: [:each | each isUndefined]].
	^changes / 2! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dataServer: aClusterDataServer
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	dataServer := aClusterDataServer.! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform an accumulation of the data from the server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	dataServer reset.
	dataSetSize := 0.
	[ dataServer atEnd]
		whileFalse:[ self accumulate: dataServer next.
					 dataSetSize := dataSetSize + 1.
					].
	^self collectChangesAndResetClusters! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Close the data server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer close! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
indexOfNearestCluster: aVector
		"Private - Answers the index of the cluster nearest to aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| distance index |
	index := 1.
	distance := ( result at: 1) distanceTo: aVector.
	2 to: result size do:
		[ :n | | x |
		  x := ( result at: n) distanceTo: aVector.
		  x < distance
			ifTrue: [ distance := x.
					  index := n.
					].
		].
	^index! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger server: aClusterDataServer type: aClusterClass
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self dataServer: aClusterDataServer.
	self clusters: ( (1 to: anInteger) collect: [ :n | aClusterClass new]).
	minimumRelativeClusterSize := 0.
	^self! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer open.
	result 
		do: [:each | each isUndefined ifTrue: [each centerOn: dataServer next]]! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimumClusterSize
	^(minimumRelativeClusterSize * dataSetSize) rounded! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimumRelativeClusterSize: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	minimumRelativeClusterSize := aNumber max: 0.! !

!DhbClusterFinder methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	aStream nextPutAll: 'Iterations: '.
	iterations printOn: aStream.
	result do: [ :each | aStream cr. each printOn: aStream].! !

DhbIterativeProcess subclass: #DhbFunctionalIterator
	instanceVariableNames: 'functionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFunctionalIterator commentStamp: '<historical>' prior: 0!
A DhbFunctionalIterator is an abstract base class for IterativeProcesses operating on a function. In the sense of these classes, a function is a block or object responding to the value: selector. 

Subclasses will redefine (in addition to methods prescribed in IterativeProcess)
	computeInitialValues 

New instances may be created with the function: class method, or existing instances may be assigned a new function with setFunction:

All functions are required to be single argument blocks, so multivariable functions must be wrapped in a way that value: takes an array of values as input.

!

DhbFunctionalIterator subclass: #DhbBisectionZeroFinder
	instanceVariableNames: 'positiveX negativeX'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	positiveX isNil
		ifTrue: [ self error: 'No positive value supplied'].
	negativeX isNil
		ifTrue: [ self error: 'No negative value supplied'].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform one step of bisection.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	result := ( positiveX + negativeX) * 0.5.
	( functionBlock value: result) > 0
		ifTrue: [ positiveX := result]
		ifFalse:[ negativeX := result].
	^self relativePrecision: ( positiveX - negativeX) abs! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
findNegativeXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ negativeX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: negativeX) < 0
		] whileFalse: [ n := n + 0.1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a negative function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
findPositiveXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ positiveX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: positiveX) > 0
		] whileFalse: [ n := n + 1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a positive function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setNegativeX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) < 0
		ifFalse:[ self error: 'Function is not negative at x = ', aNumber printString].
	negativeX := aNumber.! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setPositiveX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) > 0
		ifFalse:[ self error: 'Function is not positive at x = ', aNumber printString].
	positiveX := aNumber.! !

DhbFunctionalIterator subclass: #DhbFunctionOptimizer
	instanceVariableNames: 'optimizingPointClass bestPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super defaultPrecision * 100! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
forOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self new initializeForOptimizer: aFunctionOptimizer! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMaximizer; setFunction: aFunction! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMinimizer; setFunction: aFunction! !

!DhbFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPointAt: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: aNumber function: functionBlock)! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
bestPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^bestPoints! !

!DhbFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints first position.! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
functionBlock
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^functionBlock! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialValue: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	result := aVector copy.! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := SortedCollection sortBlock: [ :a :b | a betterThan: b].
	^super initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeAsMaximizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMaximizingPoint.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeAsMinimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMinimizingPoint.
	^self! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := aFunctionOptimizer pointClass.
	functionBlock := aFunctionOptimizer functionBlock.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointClass
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^optimizingPointClass! !

!DhbFunctionOptimizer methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	bestPoints do: [ :each | aStream cr. each printOn: aStream].! !

DhbFunctionOptimizer subclass: #DhbGeneticOptimizer
	instanceVariableNames: 'chromosomeManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^500! !

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^0! !

!DhbGeneticOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPointAt: aNumber 
	"necessary since nan's and infinites can throw off calculations. and calculations become a tiny bit more efficient"
	| p |
 	p:=optimizingPointClass vector: aNumber function: functionBlock.
 	((p value isInfinite)or:[p value isNaN]) ifFalse: [bestPoints add: p].! !

!DhbGeneticOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chromosomeManager: aChromosomeManager
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager := aChromosomeManager.
	^self! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'wernerkassens 10/29/2014 15:10'!
collectPoints
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 23-Feb-00"

	| bestPoint |
	bestPoints isEmpty ifFalse: [bestPoint := bestPoints first].
	bestPoints removeAll.
	chromosomeManager population do: [:each | self addPointAt: each].
	bestPoint ifNotNil: [:b|bestPoints add: b].
	result := bestPoints first position! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^1! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| randomScale |
	randomScale := self randomScale.
	randomScale at: randomScale size put: 1. "for floating point errors in #randomScale "
	chromosomeManager reset.
	[ chromosomeManager isFullyPopulated]
		whileFalse: [ self processRandomParents: randomScale].
	self collectPoints.
	^self computePrecision! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	chromosomeManager randomizePopulation.
	self collectPoints! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
processRandomParents: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager process: ( bestPoints at: ( self randomIndex: aNumberArray)) position
						and:  ( bestPoints at: ( self randomIndex: aNumberArray)) position.! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomIndex: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| x n |
	x := Number random.
	n := 1.
	aNumberArray do: 
		[ :each |
		  x < each
			ifTrue: [ ^n].
		  n := n + 1.
		].
	 self error: 'should not be reached'.  "the original line was there because of a bug that should be taken care of now"! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomScale
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
		"changes take care of divide by zero error"
	| norm fBest fWorst answer|
	fBest := bestPoints first value.
	fWorst := bestPoints last value.
	norm :=fBest - fWorst.
	norm =0 ifTrue: [
	norm:= 1 / bestPoints size.
	answer := bestPoints collect: [:each| norm] ] ifFalse: [
		norm := 1 / norm.
		answer := bestPoints collect: [ :each | (each value - fWorst) * norm] ].
	norm := 1 / ( answer inject: 0 into: [ :sum :each | each + sum]).
	fBest := 0.
	^answer collect: [ :each | fBest := each * norm + fBest. fBest]

! !

DhbFunctionOptimizer subclass: #DhbHillClimbingOptimizer
	instanceVariableNames: 'unidimensionalFinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHillClimbingOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "
	unidimensionalFinder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	unidimensionalFinder desiredPrecision: desiredPrecision.
	bestPoints := ( 1 to: result size)
							collect: [ :n |
									   ( DhbVectorProjectedFunction function: functionBlock)
											direction: ( ( DhbVector new: result size)
															atAllPut: 0;
															at: n put: 1;
															yourself);
											yourself
									 ].! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| oldResult |
	precision := 1.0.
	bestPoints inject: result
				 into: [ :prev :each | ( self minimizeDirection: each from: prev)].
	self shiftDirections.
	self minimizeDirection: bestPoints last.
	oldResult := result.
	result := bestPoints last origin.
	precision := 0.0.
	result with: oldResult do:
		[ :x0 :x1 |
		  precision := ( self precisionOf: (x0 - x1) abs relativeTo: x0 abs) max: precision.
		].
	^precision! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizeDirection: aVectorFunction 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^unidimensionalFinder
		reset;
		setFunction: aVectorFunction;
		addPointAt: 0;
		addPointAt: precision;
		addPointAt: precision negated;
		evaluate! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizeDirection: aVectorFunction from: aVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	^aVectorFunction
		origin: aVector;
		argumentWith: ( self minimizeDirection: aVectorFunction)! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shiftDirections
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| position delta firstDirection |
	firstDirection := bestPoints first direction.
	bestPoints inject: nil
					into: [ :prev :each |
							position isNil
								ifTrue: [ position := each origin]
								ifFalse:[ prev direction: each direction].
							each
							].
	position := bestPoints last origin - position.
	delta := position norm.
	delta > desiredPrecision
		ifTrue: [ bestPoints last direction: (position scaleBy: (1 / delta))]
		ifFalse:[ bestPoints last direction: firstDirection].! !

DhbFunctionOptimizer subclass: #DhbMultiVariableGeneralOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	self range notNil
		ifTrue: [ self performGeneticOptimization].
	self performSimplexOptimization.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| optimizer |
	optimizer := DhbHillClimbingOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision;
			  maximumIterations: maximumIterations;
			  initialValue: result.
	result := optimizer evaluate.
	^optimizer precision! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^result! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	result := anArrayOrVector.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'WernerKassens 4/15/2014 09:57'!
performGeneticOptimization
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	| optimizer manager |
	optimizer := DhbGeneticOptimizer forOptimizer: self.
	manager := DhbVectorChromosomeManager new: 100 mutation: 0.1 crossover: 0.1.
	manager origin: self origin asDhbVector; range: self range asDhbVector.
	optimizer chromosomeManager: manager.
	result := optimizer evaluate.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'WernerKassens 4/15/2014 09:57'!
performSimplexOptimization
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/29/00"

	| optimizer |
	optimizer := DhbSimplexOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision sqrt; maximumIterations: maximumIterations; initialValue: result asDhbVector.
	result := optimizer evaluate! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^self bestPoints! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	bestPoints := anArrayOrVector.! !

DhbFunctionOptimizer subclass: #DhbOneVariableFunctionOptimizer
	instanceVariableNames: ''
	classVariableNames: 'GoldenSection'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 10! !

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
goldenSection
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	GoldenSection isNil ifTrue: [GoldenSection := (3 - 5 sqrt) / 2].
	^GoldenSection! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[ bestPoints size  > 3] whileTrue: [ bestPoints removeLast].
	bestPoints size = 3
		ifTrue: [ self hasBracketingPoints
					ifFalse:[ bestPoints removeLast].
				].
	bestPoints size < 3
		ifTrue: [ ( DhbOptimizingBracketFinder forOptimizer: self) evaluate].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self precisionOf: ( ( bestPoints at: 2) position - ( bestPoints at: 3) position) abs
		   relativeTo: ( bestPoints at: 1) position abs! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'WernerKassens 5/2/2014 13:46'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self addPointAt: self nextXValue.
	bestPoints removeAt: self indexOfOuterPoint.
	^self computePrecision! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hasBracketingPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 |
	x1 := ( bestPoints at: 1) position.
	^( ( bestPoints at: 2) position - x1) * (( bestPoints at: 3) position - x1) < 0! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
indexOfOuterPoint
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| inferior superior x |
	inferior := false.
	superior := false.
	x := bestPoints first position.
	2 to: 4 do:
		[ :n |
		  ( bestPoints at: n) position < x
				ifTrue: [ inferior
							ifTrue: [ ^n].
						  inferior := true.
						]
				ifFalse:[ superior
							ifTrue: [ ^n].
						  superior := true.
						].
		].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
nextXValue
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| d3 d2 x1 |
	x1 := ( bestPoints at: 1) position.
	d2 := ( bestPoints at: 2) position - x1.
	d3 := ( bestPoints at: 3) position - x1.
	^( d3 abs > d2 abs ifTrue: [ d3]
					   ifFalse:[ d2]) * self class goldenSection + x1! !

!DhbOneVariableFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	[ bestPoints isEmpty] whileFalse: [ bestPoints removeLast].! !

DhbOneVariableFunctionOptimizer subclass: #DhbOptimizingBracketFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOptimizingBracketFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialPoints: aSortedCollection function: aFunction
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new setInitialPoints: aSortedCollection; setFunction: aFunction! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[bestPoints size < 2] whileTrue: [self addPointAt: Number random]! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 x2 |
	x1 := ( bestPoints at: 1) position.
	x2 := ( bestPoints at: 2) position.
	self addPointAt: ( x1 * 3 - ( x2 * 2)).
	precision := ( x2 - x1) * ( ( bestPoints at: 3) position - x1).
	self hasConverged
		ifFalse:[ bestPoints removeLast].
	^precision! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints.! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super initializeForOptimizer: aFunctionOptimizer.
	bestPoints := aFunctionOptimizer bestPoints.
	^self! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setInitialPoints: aSortedCollection
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := aSortedCollection.! !

DhbFunctionOptimizer subclass: #DhbSimplexOptimizer
	instanceVariableNames: 'worstVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimplexOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 1000! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
buildInitialSimplex
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	| projectedFunction finder partialResult |
	projectedFunction := DhbProjectedOneVariableFunction 
				function: functionBlock.
	finder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	finder setFunction: projectedFunction.
	[bestPoints size < (result size + 1)] whileTrue: 
			[projectedFunction
				setArgument: result;
				bumpIndex.
			partialResult := finder
						reset;
						evaluate.
			bestPoints add: (optimizingPointClass 
						vector: (projectedFunction argumentWith: partialResult)
						function: functionBlock)]! !

!DhbSimplexOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: result function: functionBlock).
	self buildInitialSimplex.
	worstVector := bestPoints removeLast position! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "
	| functionValues bestFunctionValue |
	functionValues := bestPoints collect: [ :each | each value].
	bestFunctionValue := functionValues removeFirst.
	^functionValues inject: 0
					into: [ :max :each | ( self precisionOf: ( each - bestFunctionValue) abs relativeTo: bestFunctionValue abs) max: max]! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
contract
	"Private - Contract the Simplex around the best Vector.
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	| bestVector oldVectors |
	bestVector := bestPoints first position.
	oldVectors := OrderedCollection with: worstVector.
	[bestPoints size > 1] whileTrue: [oldVectors add: bestPoints removeLast position].
	oldVectors do: [:each | self contract: each around: bestVector].
	worstVector := bestPoints removeLast position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
contract: aVector around: bestVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	bestPoints 
		add: (optimizingPointClass vector: bestVector * 0.5 + (aVector * 0.5)
				function: functionBlock)! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| centerOfGravity newPoint nextPoint |
	centerOfGravity := (bestPoints inject: ((worstVector copy)
						atAllPut: 0;
						yourself)
				into: [:sum :each | each position + sum]) * (1 / bestPoints size).
	newPoint := optimizingPointClass vector: 2 * centerOfGravity - worstVector
				function: functionBlock.
	(newPoint betterThan: bestPoints first) 
		ifTrue: 
			[nextPoint := optimizingPointClass 
						vector: newPoint position * 2 - centerOfGravity
						function: functionBlock.
			(nextPoint betterThan: newPoint) ifTrue: [newPoint := nextPoint]]
		ifFalse: 
			[newPoint := optimizingPointClass 
						vector: centerOfGravity * 0.666667 + (worstVector * 0.333333)
						function: functionBlock.
			(newPoint betterThan: bestPoints first) ifFalse: [^self contract]].
	worstVector := bestPoints removeLast position.
	bestPoints add: newPoint.
	result := bestPoints first position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	aStream cr. 
	worstVector printOn: aStream.! !

!DhbFunctionalIterator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self new setFunction: aBlock; yourself! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	self subclassResponsibility! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	functionBlock isNil ifTrue: [self error: 'No function supplied'].
	self computeInitialValues! !

!DhbFunctionalIterator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
relativePrecision: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	^self precisionOf: aNumber relativeTo: result abs! !

!DhbFunctionalIterator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setFunction: aBlock
		"Defines the function for which zeroes will be found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	( aBlock respondsTo: #value:)
		ifFalse:[ self error: 'Function block must implement the method value:'].
	functionBlock := aBlock.! !

DhbFunctionalIterator subclass: #DhbNewtonZeroFinder
	instanceVariableNames: 'derivativeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNewtonZeroFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock1 derivative: aBlock2
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^(self new) setFunction: aBlock1; setDerivative: aBlock2; yourself! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private - If no derivative has been defined, take an ad-hoc definition.
		 If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| n |
	result isNil
		ifTrue: [ result := 0].
	derivativeBlock isNil
		ifTrue: [ derivativeBlock := self defaultDerivativeBlock].
	n := 0.
	[ (derivativeBlock value: result) equalsTo: 0]
		whileTrue: [ n := n + 1.
					 n > maximumIterations
						ifTrue: [ self error: 'Function''s derivative seems to be zero everywhere'].
					 result := Number random + result].! !

!DhbNewtonZeroFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDerivativeBlock
		"Private - Answers a block computing the function's derivative by approximation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^[ :x | 5000 * ( ( functionBlock value: (x + 0.0001)) - ( functionBlock value: (x - 0.0001)))]! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Compute one step of Newton's zero finding method. Answers the estimated precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| delta |
	delta := ( functionBlock value: result) / ( derivativeBlock value: result).
	result := result - delta.
	^self relativePrecision: delta abs! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialValue: aNumber
		"Define the initial value for the iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	result := aNumber.! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setDerivative: aBlock 
	"Defines the derivative of the function for which zeroes will be found. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 7/1/99"

	| x |
	(aBlock respondsTo: #value:)
		ifFalse: [self error: 'Derivative block must implement the method value:'].
	x := result isNil
				ifTrue: [Number random]
				ifFalse: [result + Number random].
	((aBlock value: x)
		relativelyEqualsTo: (self defaultDerivativeBlock value: x)
		upTo: 1.0e-4)
		ifFalse: [self error: 'Supplied derivative is not correct'].
	derivativeBlock := aBlock! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setFunction: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super setFunction: aBlock.
	derivativeBlock := nil.! !

DhbFunctionalIterator subclass: #DhbTrapezeIntegrator
	instanceVariableNames: 'from to sum step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbTrapezeIntegrator commentStamp: '<historical>' prior: 0!
A DhbTrapezeIntegrator is the base class for integration methods. You should not use this class for real applications, since much better results can be obtained from Romberg or Simpson integration.

!

DhbTrapezeIntegrator subclass: #DhbRombergIntegrator
	instanceVariableNames: 'order points interpolator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbRombergIntegrator commentStamp: '<historical>' prior: 0!
A DhbRombergIntegrator implements the Romberg Method. This uses Richardson extrapolation on the Trapezoid method to generate a higher order estimate.

Evenly spaced samples are used. 

See also:
http://en.wikipedia.org/wiki/Romberg%27s_method

methods:

order: anInteger
	number of rows in the tableau.
	
!

!DhbRombergIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultOrder
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	^5! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super computeInitialValues.
	points := OrderedCollection new: order.
	interpolator := DhbNevilleInterpolator points: points.
	points add: 1 @ sum.! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| interpolation |
	points addLast: (points last x * 0.25) @ self higherOrderSum.
	points size < order
		ifTrue: [ ^1].
	interpolation := interpolator valueAndError: 0.
	points removeFirst.
	result := interpolation at: 1.
	^self relativePrecision: ( interpolation at: 2) abs! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	order := self class defaultOrder.
	^super initialize! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
order: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	anInteger < 2
		ifTrue: [ self error: 'Order for Romberg integration must be larger than 1'].
	order := anInteger.! !

DhbTrapezeIntegrator subclass: #DhbSimpsonIntegrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimpsonIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	| oldResult oldSum |
	iterations < 2
		ifTrue: [ self higherOrderSum.
				  ^1
				].
	oldResult := result.
	oldSum := sum.
	result := (self higherOrderSum * 4 - oldSum) / 3.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^13! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock from: aNumber1 to: aNumber2
		"Create an new instance with given parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^super new initialize: aBlock from: aNumber1 to: aNumber2! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Private - Block the constructor method for this class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self error: 'Method new:from:to: must be used'! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	step := to - from.
	sum := ( ( functionBlock value: from) + ( functionBlock value: to)) * step /2.
	result := sum.! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| oldResult |
	oldResult := result.
	result := self higherOrderSum.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
from: aNumber1 to: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	from := aNumber1.
	to := aNumber2.! !

!DhbTrapezeIntegrator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
higherOrderSum
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| x newSum |
	x := step / 2 + from.
	newSum := 0.
	[ x < to ]
		whileTrue: [ newSum := ( functionBlock value: x) + newSum.
					 x := x + step.
				   ].
	sum := ( step * newSum + sum) / 2.
	step := step / 2.
	^sum! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aBlock from: aNumber1 to: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	functionBlock := aBlock.
	self from: aNumber1 to: aNumber2.
	^self! !

DhbIterativeProcess subclass: #DhbIncompleteBetaFunctionFraction
	instanceVariableNames: 'x q1 q2 q3 numerator denominator alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFunctionFraction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"Compute and add the next term of the fraction.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	| m m2 temp |
	m := iterations + 1.
	m2 := m * 2.
	temp := m * (alpha2 - m) * x 
				/ ((q3 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	result := result * numerator * denominator.
	temp := (alpha1 + m) negated * (q1 + m) * x 
				/ ((q2 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	temp := numerator * denominator.
	result := result * temp.
	^(temp - 1) abs! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2.
	q1 := alpha1 + alpha2.
	q2 := alpha1 + 1.
	q3 := alpha1 - 1.
	^self! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	numerator := 1.
	denominator := 1 / (self limitedSmallValue: 1 - (q1 * x / q2)).
	result := denominator! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	x := aNumber.! !

DhbIterativeProcess subclass: #DhbInfiniteSeries
	instanceVariableNames: 'termServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbInfiniteSeries subclass: #DhbContinuedFraction
	instanceVariableNames: 'numerator denominator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbContinuedFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform one iteration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	| terms delta |
	terms := termServer termsAt: iterations.
	denominator := 1 / ( self limitedSmallValue: ( (terms at: 1) * denominator + (terms at: 2))).
	numerator := self limitedSmallValue: ( (terms at: 1) / numerator + (terms at: 2)).
	delta := numerator * denominator.
	result := result * delta.
	^( delta - 1) abs! !

!DhbContinuedFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the series.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	numerator := self limitedSmallValue: termServer initialTerm.
	denominator := 0.
	result := numerator! !

!DhbInfiniteSeries class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
server: aTermServer
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^self new initialize: aTermServer! !

!DhbInfiniteSeries methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform one iteration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	| delta |
	delta := termServer termAt: iterations.
	result := result + delta.
	^self precisionOf: delta abs relativeTo: result abs! !

!DhbInfiniteSeries methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aTermServer
		"Private - Assigns the object responsible to compute each term.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	termServer := aTermServer.
	^self! !

!DhbInfiniteSeries methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the series.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	result := termServer initialTerm! !

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^50! !

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbIterativeProcess class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create an instance of the class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super new initialize! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
desiredPrecision: aNumber
		"Defines the desired precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	aNumber > 0
		ifFalse: [ ^self error: 'Illegal precision: ', aNumber printString].
	desiredPrecision := aNumber.! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluate
	"Perform the iteration until either the desired precision is attained or the number of iterations exceeds the maximum.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	iterations := 0.
	self initializeIterations.
	
	[iterations := iterations + 1.
	precision := self evaluateIteration.
	self hasConverged or: [iterations >= maximumIterations]] 
			whileFalse: [].
	self finalizeIterations.
	^self result! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^self subclassResponsibility! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Perform cleanup operation if needed (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hasConverged
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	^precision <= desiredPrecision! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	desiredPrecision := self class defaultPrecision.
	maximumIterations := self class defaultMaximumIterations.
	^self! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (must be implemented by subclass when needed).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
iterations
		"Answers the number of iterations performed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^iterations! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
limitedSmallValue: aNumber
		"Private - prevent aNumber from being smaller in absolute value than a small number.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^aNumber abs < DhbFloatingPointMachine new smallNumber
			ifTrue: [ DhbFloatingPointMachine new smallNumber]
			ifFalse:[ aNumber]! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximumIterations: anInteger
		"Defines the maximum number of iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	( anInteger isInteger and: [ anInteger > 1])
		ifFalse: [ ^self error: 'Invalid maximum number of iteration: ', anInteger printString].
	maximumIterations := anInteger.! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
precision
		"Answer the attained precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^precision! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
precisionOf: aNumber1 relativeTo: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/5/99 "
	^aNumber2 > DhbFloatingPointMachine new defaultNumericalPrecision
		ifTrue: [ aNumber1 / aNumber2]
		ifFalse:[ aNumber1]! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
result
		"Answer the result of the iterations (if any)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^result! !

DhbIterativeProcess subclass: #DhbJacobiTransformation
	instanceVariableNames: 'lowerRows transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aSymmetricMatrix
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^super new initialize: aSymmetricMatrix! !

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| indices |
	indices := self largestOffDiagonalIndices.
	self transformAt: ( indices at: 1) and: ( indices at: 2).
	^precision! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
exchangeAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| temp n |
	n := anInteger + 1.
	temp := result at: n.
	result at: n put: ( result at: anInteger).
	result at: anInteger put: temp.
	transform do:
		[ :each |
		  temp := each at: n.
		  each at: n put: ( each at: anInteger).
		  each at: anInteger put: temp.
		].! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Transfer the eigenValues into a vector and set this as the result.
		 eigen values and transform matrix are sorted using a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "

	| n |
	n := 0.
	result := lowerRows collect: 
					[:each | 
					n := n + 1.
					each at: n].
	self sortEigenValues! !

!DhbJacobiTransformation methodsFor: 'initialization' stamp: 'wernerkassens 10/29/2014 14:57'!
initialize: aSymmetricMatrix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n |
	n := aSymmetricMatrix numberOfRows.
	lowerRows := Array new: n.
	transform := Array new: n.
	1 to: n do:
		[ :k |
		  lowerRows at: k put: ( ( aSymmetricMatrix rowAt: k) copyFrom: 1 to: k).
		  transform at: k put: ( ( Array new: n) atAllPut: 0; at: k put: 1; yourself).
		].
	^self! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestOffDiagonalIndices
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n m abs |
	n := 2.
	m := 1.
	precision := ( ( lowerRows at: n) at: m) abs.
	1 to: lowerRows size do:
		[ :i |
		  1 to: ( i - 1) do:
			[ :j |
			  abs := ( ( lowerRows at: i) at: j) abs.
			  abs > precision
				ifTrue: [ n := i.
						  m := j.
						  precision := abs.
						]. 
			].
		].
	^Array with: m with: n! !

!DhbJacobiTransformation methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| first |
	first := true.
	lowerRows do: 
		[ :each |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  each printOn: aStream.
		].! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sortEigenValues
		"Private - Use a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n bound m |
	n := lowerRows size.
	bound := n.
	[ bound = 0 ]
		whileFalse: [ m := 0.
					  1 to: bound - 1 do:
						[ :j |
						  ( result at: j) abs > ( result at: j + 1) abs
							ifFalse:[ self exchangeAt: j.
									  m := j.
									].
						].
					  bound := m.
					].! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
transform
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^DhbMatrix rows: transform! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
transformAt: anInteger1 and: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| d t s c tau apq app aqq arp arq |
	apq := ( lowerRows at: anInteger2) at: anInteger1.
	apq = 0
		ifTrue: [ ^nil].
	app := ( lowerRows at: anInteger1) at: anInteger1.
	aqq := ( lowerRows at: anInteger2) at: anInteger2.
	d := aqq - app.
	arp := d * 0.5 / apq.
	t := arp > 0 ifTrue: [ 1 / ( ( arp squared + 1) sqrt + arp)]
				 ifFalse:[ 1 / ( arp - ( arp squared + 1) sqrt)].
	c := 1 / ( t squared + 1) sqrt.
	s := t * c.
	tau := s / ( 1 + c).
	1 to: ( anInteger1 - 1)
		do: [ :r |
			  arp := ( lowerRows at: anInteger1) at: r.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: anInteger1) at: r put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger1 + 1) to: ( anInteger2 - 1)
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger2 + 1) to: lowerRows size
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: r) at: anInteger2.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	1 to: lowerRows size
		do: [ :r |
			  arp := ( transform at: r) at: anInteger1.
			  arq := ( transform at: r) at: anInteger2.
			  ( transform at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( transform at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( lowerRows at: anInteger1) at: anInteger1 put: ( app - (t * apq)).
	( lowerRows at: anInteger2) at: anInteger2 put: ( aqq + (t * apq)).
	( lowerRows at: anInteger2) at: anInteger1 put: 0.! !

DhbIterativeProcess subclass: #DhbLargestEigenValueFinder
	instanceVariableNames: 'matrix eigenvector transposeEigenvector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLargestEigenValueFinder class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^100! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aMatrix
		"Create a new instance of the receiver for a given matrix and default precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; yourself! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aMatrix precision: aNumber
		"Create a new instance of the receiver for a given matrix and desired precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; desiredPrecision: aNumber; yourself! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
eigenvalue
	"Answer the eigen value found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^result! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
eigenvector
	"Answer the normalized eigen vector found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^eigenvector * (1 / eigenvector norm)! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"Iterate the product of the matrix of the eigen vector and the transpose.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| oldEigenvalue |
	oldEigenvalue := result.
	transposeEigenvector := transposeEigenvector * matrix.
	transposeEigenvector := transposeEigenvector 
				* (1 / (transposeEigenvector at: 1)).
	eigenvector := matrix * eigenvector.
	result := eigenvector at: 1.
	eigenvector := eigenvector * (1 / result).
	^oldEigenvalue isNil 
		ifTrue: [2 * desiredPrecision]
		ifFalse: [(result - oldEigenvalue) abs]! !

!DhbLargestEigenValueFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aMatrix
		"Defines the matrix for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	matrix := aMatrix.! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	eigenvector := DhbVector new: matrix numberOfRows.
	eigenvector atAllPut: 1.0.
	transposeEigenvector := DhbVector new: eigenvector size.
	transposeEigenvector atAllPut: 1.0! !

!DhbLargestEigenValueFinder methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
nextLargestEigenValueFinder
	"Return an eigen value finder for the same eigen values of the receiver except the one found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	| norm |
	norm := 1 / (eigenvector * transposeEigenvector).
	^self class 
		matrix: matrix * ((DhbSymmetricMatrix identity: eigenvector size) 
						- (eigenvector * norm tensorProduct: transposeEigenvector))
		precision: desiredPrecision! !

DhbIterativeProcess subclass: #DhbLeastSquareFit
	instanceVariableNames: 'dataHolder errorMatrix chiSquare equations constants degreeOfFreedom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	^self points: aHistogram
		function: (DhbScaledProbabilityDensityFunction histogram: aHistogram
				distributionClass: aProbabilityDensityFunctionClass)! !

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'wernerkassens 10/29/2014 15:34'!
points: aDataHolder function: aParametricFunction 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/3/99"

	^aParametricFunction isNil
		ifTrue: [nil]
		ifFalse: [self new initialize: aDataHolder data: aParametricFunction]! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| f g |
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last.
	f := f first.
	constants accumulate: g * ( ( aWeightedPoint yValue - f) * aWeightedPoint weight).
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * aWeightedPoint weight).
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulateEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder pointsAndErrorsDo: [ :each | self accumulate: each].! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chiSquare
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare isNil
		ifTrue: [ self computeChiSquare].
	^chiSquare! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'WernerKassens 4/6/2014 22:36'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	errorMatrix := DhbLUPDecomposition direct: equations.
	^(errorMatrix solve: constants)ifNil:[SingularMatrixError new signal:'singular Errormatrix']! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeChiSquare
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare := 0.
	degreeOfFreedom := self numberOfFreeParameters negated.
	dataHolder pointsAndErrorsDo:
		[ :each |
		  chiSquare := ( each chi2Contribution: result) + chiSquare.
		  degreeOfFreedom := degreeOfFreedom + 1.
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	constants atAllPut: 0.
	equations do: [ :each | each atAllPut: 0].
	self accumulateEquationSystem.! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/6/00 "
	^( DhbChiSquareDistribution degreeOfFreedom: self degreeOfFreedom) confidenceLevel: self chiSquare! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	degreeOfFreedom isNil
		ifTrue: [ self computeChiSquare].
	^degreeOfFreedom! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'WernerKassens 5/2/2014 08:41'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	errorMatrix inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal:'singular Errormatrix' ] 
		ifNotNilDo: [ :i| ^DhbSymmetricMatrix rows: i ].
! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| changes maxChange|
	self computeEquationSystem.
	changes := self computeChanges.
	result changeParametersBy: changes.
	maxChange := 0.
	result parameters with: changes do: 
		[ :r :d | maxChange := ( d / r) abs max: maxChange].
	^maxChange! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	equations := nil.
	constants := nil.
	degreeOfFreedom := nil.
	chiSquare := nil! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Least square fit'! !

!DhbLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aDataHolder data: aParametricFunction
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder := aDataHolder.
	result := aParametricFunction.
	^self! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| n |
	n := self numberOfParameters.
	constants := (DhbVector new: n)
				atAllPut: 0;
				yourself.
	equations := (1 to: n) collect: 
					[:k | 
					(DhbVector new: n)
						atAllPut: 0;
						yourself]! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self numberOfParameters! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^result parameters size! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	^result value: aNumber! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	| valueGradient |
	valueGradient := result valueAndGradient: aNumber.
	^Array with: valueGradient first
		   with: ( valueGradient last * ( self errorMatrix * valueGradient last)) sqrt! !

DhbLeastSquareFit subclass: #DhbMaximumLikelihoodHistogramFit
	instanceVariableNames: 'count countVariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint
		"Private - 
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| f g temp inverseProbability|
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last copyFrom: 1 to: ( f last size - 1).
	f := f first.
	f = 0 ifTrue: [ ^nil].
	inverseProbability := 1 / f.
	temp := aWeightedPoint yValue * inverseProbability.
	constants accumulate: g * temp.
	temp := temp * inverseProbability.
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * temp).
		].! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super computeChanges copyWith: 0! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNormalization
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| numerator denominator temp |
	numerator := 0.
	denominator := 0.
	dataHolder pointsAndErrorsDo: 
			[:each | 
			temp := result value: each xValue.
			temp = 0 
				ifFalse: 
					[numerator := numerator + (each yValue squared / temp).
					denominator := denominator + temp]].
	count := ( numerator / denominator) sqrt.
	countVariance := numerator / ( 4 * count).! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Compute the normalization factor.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	self computeNormalization.
	result setCount: count.
	super finalizeIterations! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Maximum likelihood fit'! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	result setCount: 1.
	count := dataHolder totalCount.
	super initializeIterations! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalization
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^count! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalizationError
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^countVariance sqrt! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^super numberOfParameters! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super numberOfParameters - 1! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "

	| valueGradient gradient gVar |
	valueGradient := result valueAndGradient: aNumber.
	gradient := valueGradient last copyFrom: 1 to: valueGradient last size - 1.
	gVar := gradient * (self errorMatrix * gradient) / count.
	^Array with: valueGradient first
		with: ((valueGradient first / count) squared * countVariance + gVar) sqrt! !

Object subclass: #DhbLUPDecomposition
	instanceVariableNames: 'rows permutation parity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
direct: anArrayOfArrays
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	^self new initialize: anArrayOfArrays.! !

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equations: anArrayOfArrays
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
		^self new initialize: ( anArrayOfArrays collect: [ :each | each copy]).! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
backwardSubstitution: anArray
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n sum answer|
	n := rows size.
	answer := DhbVector new: n.
	n to: 1 by: -1 do:
		[ :i |
		  sum := anArray at: i.
		  ( i + 1) to: n do: [ :j | sum := sum - ( ( ( rows at: i) at: j) * ( answer at: j))].
		  answer at: i put: sum / ( ( rows at: i) at: i).
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
decompose
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n |
	n := rows size.
	permutation := (1 to: n) asArray.
	1 to: ( n - 1) do:
		[ :k |
		  self swapRow: k withRow: ( self largestPivotFrom: k);
			   pivotAt: k.
		].! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
determinant
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	| n |
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	permutation = 0
		ifTrue: [ ^0].	"Singular matrix has 0 determinant"
	n := 0.
	^rows inject: parity
		  into: [ :det :each | n := n + 1. ( each at: n) * det]! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
forwardSubstitution: anArray
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n sum answer|
	answer := permutation collect: [ :each | anArray at: each].
	n := rows size.
	2 to: n do:
		[ :i |
		  sum := answer at: i.
		  1 to: ( i - 1) do: [ :j | sum := sum - ( ( ( rows at: i) at: j) * ( answer at: j))].
		  answer at: i put: sum.
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anArrayOfArrays
		"Private - A copy of the original array is made.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	rows := anArrayOfArrays.
	parity := 1.
	^self! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'WernerKassens 5/9/2014 18:07'!
inverseMatrixComponents
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| n inverseRows column |
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	permutation = 0
		ifTrue: [ ^nil].	"Singular matrix has no inverse"
	n := rows size.
	inverseRows :=( 1 to: n) asDhbVector collect: [ :j | DhbVector new: n].
	1 to: n do:
		[ :j |
		  column := self solve: ( ( Array new: rows size) atAllPut: 0; at: j put: 1; yourself).
		  column ifNil: [^nil].	"Singular matrix has no inverse"
		  1 to: n do: [ :i | ( inverseRows at: i) at: j put: ( column at: i)].
		]. 
	^inverseRows! !

!DhbLUPDecomposition methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestPivotFrom: anInteger
		"Private - Answers the largest pivot element in column anInteger, from position anInteger upward.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| valueOfMaximum indexOfMaximum value |
	valueOfMaximum := ( ( rows at: anInteger) at: anInteger) abs.
	indexOfMaximum := anInteger.
	( anInteger + 1) to: rows size do:
		[ :n |
		  value := ( ( rows at: n) at: anInteger) abs.
		  value > valueOfMaximum
				ifTrue: [ valueOfMaximum := value.
						  indexOfMaximum := n.
						].
		].
	^indexOfMaximum! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pivotAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| inversePivot size k |
	inversePivot := 1 / ( ( rows at: anInteger) at: anInteger).
	size := rows size.
	k := anInteger + 1.
	k to: size
		do: [ :i |
			  ( rows at: i) at: anInteger put: (( rows at: i) at: anInteger) * inversePivot.
			  k to: size
				do: [ :j |
					  ( rows at: i) at: j put: ( ( rows at: i) at: j) - ( (( rows at: i) at: anInteger) * (( rows at: anInteger) at: j)).
					]
			].! !

!DhbLUPDecomposition methodsFor: 'display' stamp: 'wernerkassens 10/29/2014 14:58'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	| first delimitingString |
	first := true.
	rows do:
		[ :row |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  delimitingString := '('.
		  row do:
			[ :each |
				aStream nextPutAll: delimitingString.
				each printOn: aStream.
				delimitingString := ' '.
			].
		  aStream nextPut: $).
		].! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
protectedDecomposition
		"Private - If decomposition fails, set permutation to 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	[ self decompose] on: Error do: [ :signal | permutation := 0. signal return: nil].! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'WernerKassens 5/9/2014 18:03'!
solve: anArrayOrVector
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	^permutation = 0
		ifTrue: [ nil]
		ifFalse:[ [self backwardSubstitution: ( self forwardSubstitution: anArrayOrVector)]on: ZeroDivide do: [:e|  nil]]! !

!DhbLUPDecomposition methodsFor: 'transformation' stamp: 'wernerkassens 10/29/2014 16:16'!
swapRow: anInteger1 withRow: anInteger2
	"Private - Swap the rows indexed by the given integers.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| swappedRow |
	anInteger1 = anInteger2 ifTrue: [ ^ self ].
	swappedRow := rows at: anInteger1.
	rows at: anInteger1 put: (rows at: anInteger2).
	rows at: anInteger2 put: swappedRow.
	swappedRow := permutation at: anInteger1.
	permutation at: anInteger1 put: (permutation at: anInteger2).
	permutation at: anInteger2 put: swappedRow.
	parity := parity negated! !

Object subclass: #DhbLagrangeInterpolator
	instanceVariableNames: 'pointCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLagrangeInterpolator commentStamp: '<historical>' prior: 0!
A DhbLagrangeInterpolator takes a set of points (include each with #add:) and interpolates values using the Lagrange Interpolation Polynomial. This is the polynomial of minimum degree through all points in the pointCollection.

This is appropriate for interpolation only (interior to the region of the set of values), and sensitive to several conditions.

The method value: yields an approximation at a given value of the independent variable. Calculation is deferred until value is sent. If the set of points is fixed, NewtonInterpolator precomputes the value function. If the set of points is likely to change more often than interpolated values are needed, this is a fair choice.!

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver without points. Points must be added with add:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^super new initialize! !

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
points: aCollectionOfPoints
		"Create a new instance of the receiver with given points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: aCollectionOfPoints! !

!DhbLagrangeInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^pointCollection add: aPoint! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^OrderedCollection new! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self initialize: self defaultSamplePoints! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	pointCollection := aCollectionOfPoints.
	^self! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
size
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 3/12/00 "
	^pointCollection size! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| norm dx products answer size |
	norm := 1.
	size := pointCollection size.
	products := Array new: size.
	products atAllPut: 1.
	1 to: size
		do: [ :n |
			  dx := aNumber - ( self xPointAt: n).
			  dx = 0
				ifTrue: [ ^( self yPointAt: n)].
			  norm := norm * dx.
			  1 to: size
				do: [ :m |
					  m = n
						ifFalse:[ products at: m put: ( (( self xPointAt: m) - ( self xPointAt: n)) * ( products at: m))].
					].
			].
	answer := 0.
	1 to: size do:
		[ :n | answer := ( self yPointAt: n) / ( ( products at: n) * ( aNumber - ( self xPointAt: n))) + answer].
	^norm * answer! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
xPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) x! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
yPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) y! !

DhbLagrangeInterpolator subclass: #DhbNevilleInterpolator
	instanceVariableNames: 'leftErrors rightErrors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbNevilleInterpolator subclass: #DhbBulirschStoerInterpolator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBulirschStoerInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| diff ratio |
	ratio := ( ( self xPointAt: anInteger1) - aNumber) * ( rightErrors at: anInteger1)
							/ ( (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber).
	diff := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1))
							/ ( ratio - ( leftErrors at: ( anInteger1 + 1))).
	rightErrors at: anInteger1 put: ( leftErrors at: ( anInteger1 + 1)) * diff.	
	leftErrors at: anInteger1 put: ratio * diff.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| leftDist rightDist ratio |
	leftDist := ( self xPointAt: anInteger1) - aNumber.
	rightDist := (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber.
	ratio := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1)) / ( leftDist - rightDist).
	leftErrors at: anInteger1 put: ratio * leftDist.
	rightErrors at: anInteger1 put: ratio * rightDist.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbNevilleInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	
	super initialize.
	leftErrors := Array new.
	rightErrors := Array new! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeDifferences: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| size nearestIndex dist minDist |
	size := pointCollection size.
	leftErrors size = size
		ifFalse:[ leftErrors := Array new: size.
				  rightErrors := Array new: size.
				].
	minDist := ( ( self xPointAt: 1) - aNumber) abs.
	nearestIndex := 1.
	leftErrors at: 1 put: ( self yPointAt: 1).
	rightErrors at: 1 put: leftErrors first.
	2 to: size do:
		[ :n |
		  dist := ( ( self xPointAt: n) - aNumber) abs.
		  dist < minDist
			ifTrue: [ dist = 0
						ifTrue: [ ^n negated].
					  nearestIndex := n.
					  minDist := dist.
					].
		 leftErrors at: n put: ( self yPointAt: n).
		 rightErrors at: n put: ( leftErrors at: n).
		].
	^nearestIndex! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	^(self valueAndError: aNumber) first! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"Compute and return the interpolated value of the interpolation Lagranage polynomial
		 and its estimated error.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	| size nearestIndex answer error |
	nearestIndex := self initializeDifferences: aNumber.
	nearestIndex < 0
		ifTrue: [ ^Array with: ( self yPointAt: nearestIndex negated) with: 0].
	answer := leftErrors at: nearestIndex.
	nearestIndex := nearestIndex - 1.
	size := pointCollection size.
	1 to: ( size - 1) do:
		[ :m |
		  1 to: ( size - m) do:
			[ :n | self computeDifference: aNumber at: n order: m].
		  size - m > ( 2 * nearestIndex)
				ifTrue: [ error := leftErrors at: ( nearestIndex + 1) ]
				ifFalse:[ error := rightErrors at: ( nearestIndex).
							  nearestIndex := nearestIndex - 1.
							].
		  answer := answer + error.
		].
	^Array with: answer with: error abs! !

DhbLagrangeInterpolator subclass: #DhbNewtonInterpolator
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNewtonInterpolator commentStamp: '<historical>' prior: 0!
A DhbNewtonInterpolator is a specialized Lagrange Interpolator which precomputes the polynomial to be evaluated when interpolating. This is twice as expensive as direct evaluation, but produces a linear time (in the number of points) evaluation method for fixed points.

add: resets the coefficients

value: lazily initializes the coefficients and yields an interpolated value.

computeCoefficients and resetCoefficients should be considered private.!

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	self resetCoefficients.
	^super add: aPoint! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeCoefficients
		"Private - Computes the coefficients for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| size k1 kn|
	size := pointCollection size.
	coefficients := ( 1 to: size) collect: [ :n | self yPointAt: n].
	1 to: (size - 1)
		do: [ :n |
			  size to: ( n + 1)  by: -1
				do: [ :k |
					  k1 := k - 1.
					  kn := k - n.
					  coefficients at: k put: ( (( coefficients at: k) - ( coefficients at: k1)) 
											/ ((self xPointAt: k) - (self xPointAt: kn))).
					].
			].! !

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetCoefficients
		"Private - Reset the coefficients of the receiver to force a new computation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	coefficients := nil.! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer size |
	coefficients isNil
		ifTrue: [ self computeCoefficients].
	size := coefficients size.
	answer := coefficients at: size.
	(size - 1) to: 1 by: -1
		do: [ :n | answer := answer * ( aNumber - (self xPointAt:  n)) + ( coefficients at: n)].
	^answer! !

DhbNewtonInterpolator subclass: #DhbSplineInterpolator
	instanceVariableNames: 'startPointDerivative endPointDerivative'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSplineInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeSecondDerivatives
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| size u w s dx inv2dx |
	size := pointCollection size.
	coefficients := Array new: size.
	u := Array new: size - 1.
	startPointDerivative isNil 
		ifTrue: 
			[coefficients at: 1 put: 0.
			u at: 1 put: 0]
		ifFalse: 
			[coefficients at: 1 put: -1 / 2.
			s := 1 / (( self xPointAt: 2) x - ( self xPointAt: 1) x).
			u at: 1
				put: 3 * s 
						* (s * (( self yPointAt: size) - ( self yPointAt: size - 1)) 
								- startPointDerivative)].
	2 to: size - 1
		do: 
			[:n | 
			dx := (self xPointAt: n) - (self xPointAt: ( n - 1)).
			inv2dx := 1 / (( self xPointAt: n + 1) - (self xPointAt: n - 1)).
			s := dx * inv2dx.
			w := 1 / (s * (coefficients at: n - 1) + 2).
			coefficients at: n put: (s - 1) * w.
			u at: n
				put: (((( self yPointAt: n + 1) - ( self yPointAt: n)) 
						/ (( self xPointAt: n + 1) - ( self xPointAt: n)) 
							- ((( self yPointAt: n) - ( self yPointAt: n - 1)) / dx)) * 6 
						* inv2dx - ((u at: n - 1) * s)) 
						* w].
	endPointDerivative isNil 
		ifTrue: [coefficients at: size put: 0]
		ifFalse: 
			[w := 1 / 2.
			s := 1 / ((self xPointAt:  size) - (self xPointAt: ( size - 1))).
			u at: 1
				put: 3 * s * (endPointDerivative 
								- (s * (self yPointAt: size) - (self yPointAt: size - 1))).
			coefficients at: size
				put: s - (w * (u at: size - 1) / ((coefficients at: size - 1) * w + 1))].
	size - 1 to: 1
		by: -1
		do: 
			[:n | 
			coefficients at: n
				put: (coefficients at: n) * (coefficients at: n + 1) + (u at: n)]! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbSplineInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
endPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	endPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetEndPointDerivatives
		"Set the end point derivatives to undefined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	self setEndPointDerivatives: ( Array new: 2).! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setEndPointDerivatives: anArray
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := anArray at: 1.
	endPointDerivative := anArray at: 2.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
startPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber 
	"Computes the value of a cubic spline interpolation over the points of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| n1 n2 n step a b |
	coefficients isNil ifTrue: [self computeSecondDerivatives].
	n2 := pointCollection size.
	n1 := 1.
	[n2 - n1 > 1] whileTrue: 
			[n := (n1 + n2) // 2.
			(self xPointAt:  n) > aNumber ifTrue: [n2 := n] ifFalse: [n1 := n]].
	step := (self xPointAt: n2) - (self xPointAt: n1).
	a := ((self xPointAt: n2) - aNumber) / step.
	b := (aNumber - (self xPointAt: n1)) / step.
	^a * (self yPointAt: n1) + (b * (self yPointAt: n2)) 
		+ ((a * (a squared - 1) * (coefficients at: n1) 
				+ (b * (b squared - 1) * (coefficients at: n2))) * step squared 
				/ 6)! !

Object subclass: #DhbLanczosFormula
	instanceVariableNames: 'coefficients sqrt2Pi'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLanczosFormula commentStamp: '<historical>' prior: 0!
A DhbLanczosFormula is a singleton class which calculates an approximation to Gamma(x). 
Gamma function is a continuous extension to Factorial. Gamma(x + 1) = x * Gamma(x). For integers Gamma(n) = (n-1) factorial. 

This is called from Number>>gamma, and Number>>logGamma.

Instance variable coefficients contains the terms of the approximation.


The method is detailed in Numerical Recipes in C. The implementation is detailed in Besset's book Section 2.4
!

!DhbLanczosFormula class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance! !

!DhbLanczosFormula class methodsFor: 'instance creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	UniqueInstance := nil.! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) exp * ( self series: aNumber) * sqrt2Pi / aNumber! !

!DhbLanczosFormula methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	sqrt2Pi := ( Float pi * 2) sqrt.
	coefficients := #( 76.18009172947146 -86.50532032941677 24.01409824083091 -1.231739572450155 0.1208650973866179e-2 -0.5395239384953e-5).
	^self! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
leadingFactor: aNumber
		"Private - Answers the log of the leading factor in Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| temp |
	temp := aNumber + 5.5.
	^( temp ln * ( aNumber + 0.5) - temp)! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logGamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) + ( ( self series: aNumber) * sqrt2Pi / aNumber) ln! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
series: aNumber
		"Private - Answer the value of the series of Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| term |
	term := aNumber.
	^coefficients inject: 1.000000000190015
						into: [ :sum :each | term := term + 1. each / term + sum]! !

Object subclass: #DhbLinearEquationSystem
	instanceVariableNames: 'rows solutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equations: anArrayOfArrays constant: anArray 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self new initialize: anArrayOfArrays constants: (Array with: anArray)! !

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equations: anArrayOfArrays constants: anArrayOfConstantArrays 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self new initialize: anArrayOfArrays constants: anArrayOfConstantArrays! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
backSubstitutionAt: anInteger
		"Private - Perform the back-substitution step corresponding to the anInteger-th constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| size answer accumulator |
	size := rows size.
	answer := Array new: size.
	size to: 1 by: -1 do:
		[ :n |
		  accumulator := (rows at: n) at: (anInteger + size).
		  ( n + 1) to: size
			do: [ :m | accumulator := accumulator - ( ( answer at: m) * ( ( rows at: n) at: m))].
		  answer at: n put: ( accumulator / ( ( rows at: n) at: n)).
		].
	solutions at: anInteger put: answer.! !

!DhbLinearEquationSystem methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anArrayOfArrays constants: anArrayOfConstantArrays
		"Private - Initialize the receiver with system's matrix in anArrayOfArrays and several constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	rows := anArrayOfArrays collect: [ :each | n := n + 1. each, ( anArrayOfConstantArrays collect: [ :c | c at: n])].
	^self! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestPivotFrom: anInteger
		"Private - Answers the largest pivot element in column anInteger, from position anInteger upward.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| valueOfMaximum indexOfMaximum |
	valueOfMaximum := ( rows at: anInteger) at: anInteger.
	indexOfMaximum := anInteger.
	( anInteger + 2) to: rows size do:
		[ :n |
		  ( ( rows at: n) at: anInteger) > valueOfMaximum
				ifTrue: [ valueOfMaximum := ( rows at: n) at: anInteger.
							 indexOfMaximum := n.
						   ].
		].
	^indexOfMaximum! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pivotAt: anInteger
		"Private - Performs pivot operation with pivot element at anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| inversePivot rowPivotValue row pivotRow |
	pivotRow := rows at: anInteger.
	inversePivot := 1 / ( pivotRow at: anInteger).
	( anInteger + 1) to: rows size do:
		[ :n |
		  row := rows at: n.
		  rowPivotValue := ( row at: anInteger) * inversePivot.
		  anInteger to: row size do:
			[ :m |
			  row at: m put: ( ( row at: m) - (( pivotRow at: m) * rowPivotValue)).
			].
		].! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pivotStepAt: anInteger
		"Private - Performs an optimum pivot operation at anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	self swapRow: anInteger withRow: ( self largestPivotFrom: anInteger);
		   pivotAt: anInteger.! !

!DhbLinearEquationSystem methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	| first delimitingString n k |
	n := rows size.
	first := true.
	rows do:
		[ :row |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  delimitingString := '('.
		  k := 0.
		  row do:
			[ :each |
				aStream nextPutAll: delimitingString.
				each printOn: aStream.
				k := k + 1.
				delimitingString := k < n ifTrue: [ ' '] ifFalse: [ ' : '].
			].
		  aStream nextPut: $).
		].! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
solution
		"Answers the solution corresponding to the first constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^self solutionAt: 1! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
solutionAt: anInteger
		"Answer the solution corresponding to the anInteger-th constant array.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	"solutions isNil
		ifTrue: [ [self solve] when: Error do: [ :signal |solutions := 0. signal return: nil.] ].
	solutions = 0
		ifTrue: [ ^nil].
	( solutions at: anInteger) isNil
		ifTrue: [ self backSubstitutionAt: anInteger].
	^solutions at: anInteger"

	solutions isNil
		ifTrue: [ [self solve] on: Error do: [ :signal |solutions := 0. signal return: nil.] ].
	solutions = 0
		ifTrue: [ ^nil].
	( solutions at: anInteger) isNil
		ifTrue: [ self backSubstitutionAt: anInteger].
	^solutions at: anInteger! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
solve
		"Private - Perform LU decomposition of the system.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	1 to: rows size do: [ :n | self pivotStepAt: n].
	solutions := Array new: ( (rows at: 1) size - rows size).! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'wernerkassens 10/29/2014 15:18'!
swapRow: anInteger1 withRow: anInteger2
	"Private - Swap the rows indexed by the given integers.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	| swappedRow |
	anInteger1 = anInteger2
		ifTrue: [ ^ self ].
	swappedRow := rows at: anInteger1.
	rows at: anInteger1 put: (rows at: anInteger2).
	rows at: anInteger2 put: swappedRow! !

Object subclass: #DhbLinearRegression
	instanceVariableNames: 'sum1 sumX sumY sumXX sumYY sumXY slope intercept correlationCoefficient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLinearRegression class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^( super new) reset; yourself! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	self add: aPoint weight: 1.! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint weight: aNumber 
	"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 + aNumber.
	sumX := sumX + (aPoint x * aNumber).
	sumY := sumY + (aPoint y * aNumber).
	sumXX := sumXX + (aPoint x squared * aNumber).
	sumYY := sumYY + (aPoint y squared * aNumber).
	sumXY := sumXY + (aPoint x * aPoint y * aNumber).
	self resetResults! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asEstimatedPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^( DhbEstimatedPolynomial coefficients: self coefficients)
			errorMatrix: self errorMatrix;
			yourself! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^DhbPolynomial coefficients: self coefficients! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
coefficients
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^Array with: self intercept with: self slope! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeResults
	"Private - Compute the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	| xNorm xyNorm |
	xNorm := sumXX * sum1 - (sumX * sumX).
	xyNorm := sumXY * sum1 - (sumX * sumY).
	slope := xyNorm / xNorm.
	intercept := (sumXX * sumY - (sumXY * sumX)) / xNorm.
	correlationCoefficient := xyNorm 
				/ (xNorm * (sumYY * sum1 - (sumY * sumY))) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
correlationCoefficient
		"Answers the correlation coefficient of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	correlationCoefficient isNil
		ifTrue: [ self computeResults].
	^correlationCoefficient! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix
	"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "

	| c1 cx cxx |
	c1 := 1.0 / (sumXX * sum1 - sumX squared).
	cx := sumX negated * c1.
	cxx := sumXX * c1.
	c1 := sum1 * c1.
	^DhbSymmetricMatrix rows: (Array with: (Array with: cxx with: cx)
				with: (Array with: cx with: c1))! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorOnIntercept
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sumXX / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorOnSlope
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sum1 / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
intercept
		"Answers the intercept of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	intercept isNil
		ifTrue: [ self computeResults].
	^intercept! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
remove: aPoint 
	"Remove aPoint which was accumulated into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 - 1.
	sumX := sumX - aPoint x.
	sumY := sumY - aPoint y.
	sumXX := sumXX - aPoint x squared.
	sumYY := sumYY - aPoint y squared.
	sumXY := sumXY - (aPoint x * aPoint y).
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"Set all accumulators of the receiver to zero and reset its results.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := 0.
	sumX := 0.
	sumY := 0.
	sumXX := 0.
	sumYY := 0.
	sumXY := 0.
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetResults
		"Private - Reset the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope := nil.
	intercept := nil.
	correlationCoefficient := nil.! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
slope
		"Answers the slope of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope isNil
		ifTrue: [ self computeResults].
	^slope! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answer the value interpolated at aNumber by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^aNumber * self slope + self intercept! !

Object subclass: #DhbMahalanobisCenter
	instanceVariableNames: 'center inverseCovariance accumulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new initialize: anInteger! !

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
onVector: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new center: aVector! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
center: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: aVector size.
	center := aVector.
	inverseCovariance := DhbSymmetricMatrix identity: aVector size.
	^self! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeParameters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.
	inverseCovariance := accumulator covarianceMatrix inverse.! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
count
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	| delta |
	delta := aVector - center.
	^delta * inverseCovariance * delta! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: anInteger.
	^self! !

!DhbMahalanobisCenter methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Leave center and inverse covariant matrix untouched
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator reset.! !

Object subclass: #DhbMatrix
	instanceVariableNames: 'rows lupDecomposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMatrix commentStamp: 'wernerkassens 5/1/2015 12:13' prior: 0!
a DhbMatrix is made via #rows: like this:
DhbMatrix rows: #((1 2 3)(4 5 6))."-->"
" a DhbVector(1 2 3)
   a DhbVector(4 5 6)"
and understands the usual matrix operations.
!

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
join: anArrayOfMatrices
		"Inverse of the split operation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| rows n row rowSize n1 n2 |
	rows := OrderedCollection new.
	n1 := ( anArrayOfMatrices at: 1) numberOfColumns.
	n2 := n1 + 1.
	rowSize := n1 + ( anArrayOfMatrices at: 2) numberOfColumns.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 2) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	n := 0.
	( anArrayOfMatrices at: 3) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 4) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	^self rows: rows! !

!DhbMatrix class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
lupCRLCriticalDimension
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^40! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"Create an empty square matrix of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: anInteger! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rows: anArrayOrVector
		"Create a new matrix with given components.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initializeRows: anArrayOrVector! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
* aNumberOrMatrixOrVector
		"Answers the product of the receiver with the argument.
		 The argument can be a number, matrix or vector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aNumberOrMatrixOrVector productWithMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
+ aMatrix
		"Answers the sum of the receiver with aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix addWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'WernerKassens 4/13/2014 17:11'!
- aMatrix
		"Answers the difference between the receiver and aMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aMatrix subtractWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'comparing' stamp: 'wernerkassens 11/7/2014 12:24'!
= aNumberOrMatrix 
	^(aNumberOrMatrix species = self species) and: [self rows = aNumberOrMatrix rows]! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 5/2/2014 09:31'!
adaptToNumber: rcvr andSend: selector
"selector must obviously be commutative for this simple solution, but at the moment its only used for multiplication"
^self perform:  selector with: rcvr.! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/12/2014 16:56'!
addWithRegularMatrix: aMatrix
		"Answers the sum of the receiver with aMatrix as a DhbMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^DhbMatrix rows: ( self rowsCollect: [ :each | n := n + 1. each + ( aMatrix rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/12/2014 16:57'!
addWithSymmetricMatrix: aMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/28/99 "
	^aMatrix addWithRegularMatrix: self ! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asSymmetricMatrix
		"Convert the receiver to a symmetric matrix (no check is made).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^DhbSymmetricMatrix rows: rows! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atColumn: aColumnIndex put: aCollection

	aCollection withIndexDo: [: value : rowIndex |
		self rowAt: rowIndex columnAt: aColumnIndex put: value ]
	
! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atRow: rowIndex put: aCollection

	aCollection withIndexDo: [: value : columnIndex |
		self rowAt: rowIndex columnAt: columnIndex put: value ]
	
! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
columnAt: anInteger
		"Answers the anInteger-th column of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows collect: [ :each | each at: anInteger]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
columnsCollect: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^rows last collect: [ :each | n := n + 1. aBlock value: ( self columnAt: n)]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
columnsDo: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^rows last do: [ :each | n := n + 1. aBlock value: ( self columnAt: n)]! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
determinant
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^self lupDecomposition determinant! !

!DhbMatrix methodsFor: 'comparing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hash
	^rows hash! !

!DhbMatrix methodsFor: 'initialization' stamp: 'WernerKassens 4/15/2014 09:57'!
initialize: anInteger
		"Build empty components for a square matrix.
		  No check is made: components are assumed to be orgainized in rows.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows := ( 1 to: anInteger) asDhbVector collect: [ :each | DhbVector new: anInteger].! !

!DhbMatrix methodsFor: 'initialization' stamp: 'WernerKassens 4/15/2014 09:57'!
initializeRows: anArrayOrVector
		"Defines the components of the recevier.
		  No check is made: components are assumed to be orgainized in rows.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows := anArrayOrVector asDhbVector collect: [ :each | each asDhbVector].! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverse
	"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^self isSquare 
		ifTrue: [self lupInverse]
		ifFalse: [self squared inverse * self transpose]! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inversePureCRL
		"Answer the inverse of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/5/99 "
	^self squared inversePureCRL * self transpose! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isSquare
		"Answers true if the number of rows is equal to the number of columns.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows size = rows last size! !

!DhbMatrix methodsFor: 'information' stamp: 'WernerKassens 4/8/2014 18:11'!
isSymmetric
	^self = self transpose ! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestPowerOf2SmallerThan: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved
		 Initial code: 21/3/99 "
	| m m2|
	m := 2.
	[ m2 := m * 2.
	  m2 < anInteger] whileTrue:[ m := m2].
	^m! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
lupDecomposition
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
	lupDecomposition isNil
		ifTrue: [ lupDecomposition :=DhbLUPDecomposition equations: rows].
	^lupDecomposition! !

!DhbMatrix methodsFor: 'operation' stamp: 'WernerKassens 5/2/2014 08:46'!
lupInverse
	self lupDecomposition inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal ] 
		ifNotNilDo: [ :i| ^self class rows:i ].
! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negate
		"Inverse the sign of all components of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	rows do: [ :each |each negate].! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfColumns
		"Answer the number of rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows last size! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfRows
		"Answer the number of rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows size! !

!DhbMatrix methodsFor: 'printing' stamp: 'wernerkassens 4/29/2015 17:02'!
printOn: aStream
	| first |
	(rows isNil or: [rows first isNil])
		ifTrue:[super printOn: aStream. aStream nextPutAll:'(uninitialized)'. ^self].
	first := true.
	rows do: 
		[ :each |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  each printOn: aStream.
		].! !

!DhbMatrix methodsFor: 'private' stamp: 'WernerKassens 4/12/2014 14:06'!
privateTranspose
^self transpose ! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithMatrix: aMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self productWithMatrixFinal: aMatrix! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/12/2014 14:11'!
productWithMatrixFinal: aMatrix
		"Answers the product of aMatrix with the receiver (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
		"speed optimized"
	|t|
	t :=self privateTranspose .
	^DhbMatrix  rows: ( aMatrix rowsCollect: [ :row | t rowsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/12/2014 14:07'!
productWithTransposeMatrix: aMatrix
		"Answers the product of the receiver with the transpose of aMatrix(in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^DhbMatrix rows: ( self rowsCollect: [ :row | aMatrix rowsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers the product of the receiver with aVector
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self columnsCollect: [ :each | each * aVector]! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowAt: anInteger
		"Answers the anInteger-th row of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows at: anInteger! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowAt: aRowIndex columnAt: aColumnIndex
		"Answers the aRowIndex-th, aColumnIndex-th entry in the receiver.
		 (c) Copyrights Joseph WHITESELL, 2001, all rights reserved.
		 Initial code: 08/17/2001 "
	^(rows at: aRowIndex) at: aColumnIndex! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowAt: aRowIndex columnAt: aColumnIndex put: aValue
	
	^(rows at: aRowIndex) at: aColumnIndex put: aValue! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rows
	^rows! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowsCollect: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows collect: aBlock! !

!DhbMatrix methodsFor: 'iterators' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowsDo: aBlock
		"Perform the collect: operation on the rows of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^rows do: aBlock! !

!DhbMatrix methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
scaleBy: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/11/00 "
	rows do: [ :each | each scaleBy: aNumber].! !

!DhbMatrix methodsFor: 'private' stamp: 'WernerKassens 4/9/2014 15:54'!
species
^DhbMatrix ! !

!DhbMatrix methodsFor: 'transformation' stamp: 'WernerKassens 4/15/2014 09:57'!
split
		"Private - Answers an array of 4 matrices split from the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| n m n1 m1 |
	n := self numberOfRows.
	m := self numberOfColumns.
	n1 := self largestPowerOf2SmallerThan: n.
	m1 := self largestPowerOf2SmallerThan: m.
	^Array
		with: ( self class rows: ( ( 1 to: n1) asDhbVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( 1 to: n1) asDhbVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))
		with: ( self class rows: ( ( (n1 + 1) to: n) asDhbVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( (n1 + 1) to: n) asDhbVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))! !

!DhbMatrix methodsFor: 'operation' stamp: 'wernerkassens 4/29/2015 22:21'!
squared
	|transposed|
	transposed :=self privateTranspose .
	^DhbSymmetricMatrix 
		new: transposed numberOfRows 
		function: [ :x :y|(transposed rowAt: x) * (transposed rowAt: y) ]
! !

!DhbMatrix methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
strassenProductWithMatrix: aMatrix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| matrixSplit selfSplit p1 p2 p3 p4 p5 p6 p7 |
	( self numberOfRows > 2 and: [ self numberOfColumns > 2])
		ifFalse:[ ^self class rows: ( aMatrix rowsCollect: [ :row | self columnsCollect: [ :col | row * col]])].
	selfSplit := self split.
	matrixSplit := aMatrix split.
	p1 := ( ( selfSplit at: 2) - ( selfSplit at: 4)) strassenProductWithMatrix: ( matrixSplit at: 1).
	p2 := ( selfSplit at: 4) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 2)).
	p3 := ( selfSplit at: 1) strassenProductWithMatrix: ( ( matrixSplit at: 3) + ( matrixSplit at: 4)).
	p4 := ( ( selfSplit at: 3) - ( selfSplit at: 1)) strassenProductWithMatrix: ( matrixSplit at: 4).
	p5 := ( ( selfSplit at: 1) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 4)).
	p6 := ( ( selfSplit at: 3) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 2) - ( matrixSplit at: 4)).
	p7 := ( ( selfSplit at: 1) + ( selfSplit at: 2)) strassenProductWithMatrix: ( ( matrixSplit at: 1) - ( matrixSplit at: 3)).
	^self class join: ( Array
							with: ( p5 + p4 + p6 - p2)
							with: (p1 + p2)
							with: ( p3 + p4)
							with: ( p5 + p1 - p3 - p7)
							)! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/13/2014 17:10'!
subtractWithRegularMatrix: aMatrix
		"Answers the difference between aMatrix and the receiver as aDhbMatrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| n |
	n := 0.
	^DhbMatrix rows: ( aMatrix rowsCollect: [ :each | n := n + 1. each - ( self rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/13/2014 17:14'!
subtractWithSymmetricMatrix: aMatrix
		"Answers the difference between aMatrix and the receiver.
		(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/28/99 "
	^self subtractWithRegularMatrix: aMatrix ! !

!DhbMatrix methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
transpose
		"Answer a new matrix, transpose of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self class rows: ( self columnsCollect: [ :each | each])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'WernerKassens 4/12/2014 14:08'!
transposeProductWithMatrix: aMatrix
		"Answers the product of the transpose of the receiver with aMatrix (in this order).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
		"speed optimized"
		|t|
		t :=aMatrix privateTranspose .
	^DhbMatrix  rows: ( self columnsCollect: [ :row | t rowsCollect: [ :col | row * col]])! !

DhbMatrix subclass: #DhbSymmetricMatrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSymmetricMatrix commentStamp: 'wernerkassens 5/1/2015 12:19' prior: 0!
DhbSymmetricMatrix can be made like DhbMatrix via #rows:, but the user has to make sure that this matrix is indeed symmetric as this is not internally checked for speed reasons.!

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 11:59'!
identity: anInteger
		"Create an identity matrix of dimension anInteger"
	^self new initializeIdentity: anInteger! !

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 11:59'!
join: anArrayOfMatrices
		"Inverse of the split operation"
	| rows n |
	rows := OrderedCollection new.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: each, ( ( anArrayOfMatrices at: 3) columnAt: n).
		].
	n := 0.
	( anArrayOfMatrices at: 2) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: ( ( anArrayOfMatrices at: 3) rowAt: n), each.
		].
	^self rows: rows! !

!DhbSymmetricMatrix class methodsFor: 'information' stamp: 'wernerkassens 5/1/2015 12:00'!
lupCRLCriticalDimension
	^36! !

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 10:12'!
new: dim function: aBlock
"Answer a new symmetric matrix with dimensions dim filled with 
aBlock value: RowposRespectivelyColpos value: ColposRespectivelyRowpos"
	|a aRow|
	a:=self new basicInitialize: dim.
	aRow := DhbVector new: dim.
	1 to: dim do:[:i|aRow at: i put: (aBlock value: i value:1)].
	a rows at: 1 put: aRow.
	2 to: dim do: [:rowPos|
		aRow :=DhbVector new: dim .
		1 to: rowPos -1 do:[:pos| aRow at: pos put: ((a rows at: pos) at: rowPos)].
		rowPos to: dim do:[:pos| aRow at: pos put: (aBlock value: rowPos value:pos)]. 
		(a rows) at: rowPos put: aRow ].    
	^a ! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:52'!
+ aMatrix
		"Answers the sum of the receiver with aMatrix."
	^aMatrix addWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:52'!
- aMatrix
		"Answers the difference between the receiver and aMatrix"
	^aMatrix subtractWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'double dispatching' stamp: 'wernerkassens 5/1/2015 12:22'!
addWithSymmetricMatrix: aMatrix
^self class 
	new: aMatrix numberOfRows 
	function: [ :x :y|((rows at:x)at:y)+((aMatrix rows at:x)at:y) ]
! !

!DhbSymmetricMatrix methodsFor: 'transformation' stamp: 'WernerKassens 4/8/2014 22:36'!
asSymmetricMatrix
"simple speed up"
^self! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'WernerKassens 5/2/2014 18:31'!
atColumn: aColumnIndex put: aCollection
self shouldNotImplement ! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'WernerKassens 5/2/2014 18:32'!
atRow: aRowIndex put: aCollection
self shouldNotImplement ! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'wernerkassens 5/1/2015 12:51'!
basicInitialize: anInteger
"private"
	rows := DhbVector new: anInteger.! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
clear
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/11/00 "
	rows do: [ :each | each atAllPut: 0].! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'WernerKassens 4/12/2014 14:15'!
columnAt: anInteger
"optimized "
	^rows at: anInteger   ! !

!DhbSymmetricMatrix methodsFor: 'iterators' stamp: 'WernerKassens 4/12/2014 14:16'!
columnsCollect: aBlock
		"optimized "
	^rows  collect: aBlock ! !

!DhbSymmetricMatrix methodsFor: 'iterators' stamp: 'WernerKassens 4/12/2014 14:16'!
columnsDo: aBlock
		"optimized"
	^rows do: aBlock ! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'WernerKassens 4/13/2014 11:51'!
crlInverse
"corrected sequence of calcs; s1in matrices at:2 has to be calced first (split returns matrices in this order: b, d, c).
and  taken care of degenerate cases (which is unnecessary, if this method is not called outside of inverse)"
	| matrices b1 cb1ct cb1 |
	rows size = 1
	ifTrue: [ ^self inverse1By1].
	rows size = 2
	 ifTrue: [ ^self inverse2By2].
	matrices := self split.
	b1 := (matrices at: 1) inverse.
	cb1 := (matrices at: 3) * b1.
	cb1ct := (cb1 productWithTransposeMatrix: (matrices at: 3))asSymmetricMatrix .
	matrices at: 2 put: ((matrices at: 2) - cb1ct ) inverse.
	matrices at: 3 put: (matrices at: 2) * cb1.
	matrices at: 1 put: ( b1 + (cb1 transposeProductWithMatrix: (matrices at: 3))asSymmetricMatrix).
	(matrices at: 3) negate.
	^self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'wernerkassens 5/1/2015 12:51'!
initializeIdentity: anInteger
		"Build components for an identity matrix."
	rows := ( 1 to: anInteger) asDhbVector collect: [ :n | (DhbVector new: anInteger) atAllPut: 0; at: n put: 1; yourself].! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:52'!
inverse
	"Answer the inverse of the receiver"

	^(rows size < self class lupCRLCriticalDimension or: [lupDecomposition notNil]) 
			ifTrue: [self lupInverse]
			ifFalse: [self crlInverse]! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:53'!
inverse1By1
		"Private - Answer the inverse of the receiver when it is a 1x1 matrix (no check is made)."
	^self class rows: ( DhbVector with: ( DhbVector with: ( 1/ ((rows at: 1) at: 1))))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:53'!
inverse2By2
		"Private - Answer the inverse of the receiver when it is a 2x2 matrix (no check is made)."
	| line1 line2 |
	line1 := DhbVector with: ((rows at: 2) at: 2) with: ((rows at: 1) at: 2) negated.
	line2 := DhbVector with: ((rows at: 1) at: 2) negated with: ((rows at: 1) at: 1).
	^self class rows: ( DhbVector with: line1 with: line2) * (1 / (((rows at: 1) at: 1) * ((rows at: 2) at: 2) - ((rows at: 1) at: 2) squared))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:53'!
inversePureCRL
		"Answer the inverse of the receiver."
	| matrices b1 cb1 cb1ct |
	[rows size = 1
		ifTrue: [ ^self inverse1By1]]on: ZeroDivide do: [:e| SingularMatrixError new signal].
	rows size = 2
		ifTrue: [ ^self inverse2By2].
	matrices := self split.
	b1 := ( matrices at: 1) inversePureCRL.
	cb1 := ( matrices at: 3) * b1.
	cb1ct := ( cb1 productWithTransposeMatrix: ( matrices at: 3)) asSymmetricMatrix.
	matrices at: 2 put: ( ( matrices at: 2) - cb1ct) inversePureCRL.
	matrices at: 3 put: ( matrices at: 2) * cb1.
	matrices at: 1 put: ( b1 + ( cb1 transposeProductWithMatrix: ( matrices at: 3))asSymmetricMatrix).
	( matrices at: 3) negate.
	^self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'wernerkassens 5/1/2015 12:53'!
inversePureLUP
		"Answer the inverse of the receiver."
	lupDecomposition := nil.
	self lupDecomposition inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal ] 
		ifNotNilDo: [ :i| ^self class rows:i ].
! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'wernerkassens 5/1/2015 11:57'!
isSquare
		"Answers true because a symmetric matrix is square."
	^true! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'wernerkassens 5/1/2015 11:57'!
isSymmetric
		"Answers true because the receiver is a symmetric matrix"
	^true! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'wernerkassens 5/1/2015 21:57'!
numberOfColumns
		"Answer the number of columns of the receiver. optimized"
	^rows  size! !

!DhbSymmetricMatrix methodsFor: 'private ' stamp: 'WernerKassens 4/12/2014 14:03'!
privateTranspose 
^self ! !

!DhbSymmetricMatrix methodsFor: 'transformation' stamp: 'wernerkassens 5/1/2015 11:58'!
split
	"Private - 
	Answers an array of 3 matrices split from the receiver"

	| n |
	n := self largestPowerOf2SmallerThan: rows size.
	^Array
		with: (self class rows: ((1 to: n) asDhbVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))
		with: (self class rows: ((n + 1 to: rows size) asDhbVector collect: [:k | (rows at: k)
						copyFrom: n + 1 to: rows size]))
		with: (self class superclass rows: ((n + 1 to: rows size) asDhbVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))! !

!DhbSymmetricMatrix methodsFor: 'double dispatching' stamp: 'wernerkassens 5/1/2015 12:24'!
subtractWithSymmetricMatrix: aMatrix
^self class 
	new: aMatrix numberOfRows 
	function: [ :x :y|((aMatrix rows at:x)at:y)-(( rows at:x)at:y) ]! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'WernerKassens 4/12/2014 14:05'!
transpose
"Answer a new matrix, transpose of the receiver. optimized"
	^self deepCopy! !

Object subclass: #DhbMinimizingPoint
	instanceVariableNames: 'value position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbMinimizingPoint subclass: #DhbMaximizingPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value > anOptimizingPoint value! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aVector value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^self new vector: aVector; value: aNumber; yourself! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
vector: aVector function: aFunction 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^self new: aVector value: (aFunction value: aVector)! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value < anOptimizingPoint value! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
position
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^position! !

!DhbMinimizingPoint methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "

	position printOn: aStream.
	aStream
		nextPut: $:;
		space.
	value printOn: aStream! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	value := aNumber.! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
vector: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	position := aVector! !

Object subclass: #DhbMitchellMooreGenerator
	instanceVariableNames: 'randoms lowIndex highIndex'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
constants: anArray lowIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^super new initialize: anArray lowIndex: anInteger! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
default
	"Private- 
	 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator new.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
generateSeeds: congruentialGenerator
	"Private- "
	^self constants: ((1 to: 55) collect: [:n | congruentialGenerator floatValue])
		lowIndex: 24! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := self default].
	^UniqueInstance! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset: anInteger
	"Reset the unique instance used for the default series"
	UniqueInstance := self seed: anInteger.! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
seed: anInteger
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator seed: anInteger.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
floatValue
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| x |
	x := (randoms at: lowIndex) + (randoms at: highIndex).
	x < 1.0 ifFalse: [x := x - 1.0].
	randoms at: highIndex put: x.
	highIndex := highIndex + 1.
	highIndex > randoms size ifTrue: [highIndex := 1].
	lowIndex := lowIndex + 1.
	lowIndex > randoms size ifTrue: [lowIndex := 1].
	^x! !

!DhbMitchellMooreGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anArray lowIndex: anInteger
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	randoms := anArray.
	lowIndex := anInteger.
	highIndex := randoms size.
	^self! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerValue: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^( self floatValue * anInteger) truncated! !

Object subclass: #DhbPolynomial
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomial commentStamp: '<historical>' prior: 0!
A DhbPolynomial represents a single variable polynomial function as an array of coefficients.

The constructor coefficients: anArray expects coefficients in increasing degree, that is constant term first. So, for example, to represent X^2 + 2X, we want to create:

DhbPolynomial coefficients: #(0 2 1)

The printOn: method similarly displays the polynomial from lowest nonzero degree to highest. 

DhbPolynomial coefficients: #(0 2 1)   2 X +  X^2 

!

DhbPolynomial subclass: #DhbEstimatedPolynomial
	instanceVariableNames: 'errorMatrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'WernerKassens 4/15/2014 09:57'!
error: aNumber
		"Compute the error on the value of the receiver for argument aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| errorVector term nextTerm |
	nextTerm := 1.
	errorVector := ( coefficients collect: [ :each | term := nextTerm. nextTerm := aNumber * nextTerm. term]) asDhbVector.
	^( errorVector * errorMatrix * errorVector) sqrt! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "
	^errorMatrix! !

!DhbEstimatedPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix: aMatrix
		"Defines the error matrix of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	errorMatrix := aMatrix.! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/5/99 "
	^Array with: ( self value: aNumber) with: ( self error: aNumber)! !

!DhbPolynomial class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
coefficients: anArray
		" Creates a new instance with given coefficients
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^self new initialize: anArray reverse! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
* aNumberOrPolynomial
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aNumberOrPolynomial timesPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
+ aNumberOrPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aNumberOrPolynomial addPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
- aNumberOrPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aNumberOrPolynomial subtractToPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
/ aNumberOrPolynomial
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aNumberOrPolynomial dividingPolynomial: self! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'wernerkassens 11/7/2014 12:39'!
= aNumberOrPolynomial 
	^(aNumberOrPolynomial class = self class) and: [self coefficients = aNumberOrPolynomial coefficients]! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
adaptToNumber: rcvr andSend: selector
	^(self class coefficients: (Array with: rcvr) ) perform: selector with: self! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addNumber: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	| newCoefficients |
	newCoefficients := coefficients reverse.
	newCoefficients at: 1 put: newCoefficients first + aNumber.
	^self class coefficients: newCoefficients! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) + ( self at: n)])! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
at: anInteger
		"Answers the coefficient of order anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^anInteger < coefficients size
		ifTrue: [ coefficients at: ( coefficients size - anInteger)]
		ifFalse:[ 0]! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
coefficients
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 21/5/99"

	^coefficients reverse! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
deflatedAt: aNumber
		"Answers a new polynomial quotient of the receiver with polynomial (X-aNumber)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	| remainder next newCoefficients|
	remainder := 0.
	newCoefficients := coefficients collect:
						[ :each |
						  next := remainder. 
						  remainder := remainder * aNumber + each.
						  next].
	^self class coefficients: ( newCoefficients copyFrom: 2 to: newCoefficients size) reverse! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degree
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^coefficients size - 1! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
derivative
		"Answer a new polynomial, derivative of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	| n |
	n := coefficients size.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each * n]) reverse copyFrom: 2 to: coefficients size)! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^( self dividingPolynomialWithRemainder: aPolynomial) first! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomialWithRemainder: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	| remainderCoefficients quotientCoefficients n m norm quotientDegree |
	n := self degree.
	m := aPolynomial degree.
	quotientDegree := m - n.
	quotientDegree < 0
		"this is a bugfix: variable sized instance : Behavior basicNew"
		ifTrue: [ ^Array with: ( self class coefficients: #(0)) with: aPolynomial].
	quotientCoefficients := Array new: quotientDegree + 1.
	remainderCoefficients := ( 0 to: m) collect: [ :k | aPolynomial at: k].
	norm := 1 / coefficients first.
	quotientDegree to: 0 by: -1
		do: [ :k | | x |
			  x := ( remainderCoefficients at: n + k + 1) * norm.
			  quotientCoefficients at: (quotientDegree + 1 - k) put: x.
			  (n + k - 1) to: k by: -1
				do: [ :j | 
				remainderCoefficients at: j + 1 put: 
							( ( remainderCoefficients at: j + 1) - ( x * (self at: j - k)))
				].
			].
	^Array with: ( self class coefficients: quotientCoefficients reverse)
		   with: ( self class coefficients: ( remainderCoefficients copyFrom: 1 to: n))! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hash
	^coefficients hash! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"initialize an empty polynomial as the constant zero"
	super initialize.
	coefficients := #( 0 ).
	^ self! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anArray
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	coefficients := anArray.
	^self! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integral
		"Answer a new polynomial, integral of the receiver with value 0 at x=0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^self integral: 0! !

!DhbPolynomial methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integral: aValue
		"Answer a new polynomial, integral of the receiver with given value at x=0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	| n |
	n := coefficients size + 1.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each / n]) copyWith: aValue) reverse! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isZero
	^ self coefficients allSatisfy: [:ea | ea = 0].! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negated
	^ self * -1.! !

!DhbPolynomial methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream 
	"Append to aStream a written representation of the receiver. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/1/99"

	| n firstNonZeroCoefficientPrinted |
	n := 0.
	firstNonZeroCoefficientPrinted := false.
	coefficients
		reverseDo: 
			[:each | 
			each = 0
				ifFalse: 
					[firstNonZeroCoefficientPrinted
						ifTrue: 
							[aStream space.
							((each respondsTo: #< ) and: [each < 0]) ifFalse: [aStream nextPut: $+].
							aStream space]
						ifFalse: [firstNonZeroCoefficientPrinted := true].
					(each = 1 and: [n > 0])
						ifFalse: [each printOn: aStream].
					n > 0
						ifTrue: 
							[aStream nextPutAll: ' X'.
							n > 1
								ifTrue: 
									[aStream nextPut: $^.
									n printOn: aStream]]].
			n := n + 1].
		
	"add a quick check, print a zero if the polynomial has all zero coefficients
	Daniel Uber 11 May 2012"
	firstNonZeroCoefficientPrinted ifFalse: [0 printOn: aStream].! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reciprocal
	^ (DhbPolynomial coefficients: #(1)) / self! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
roots
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	^self roots: DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
roots: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	| pol roots x rootFinder |
	rootFinder := DhbNewtonZeroFinder new.
	rootFinder desiredPrecision: aNumber.
	pol := self class coefficients: ( coefficients reverse collect: [ :each | each asFloat]).
	roots := OrderedCollection new: self degree.
	[ rootFinder setFunction: pol; setDerivative: pol derivative.
	  x := rootFinder evaluate.
	  rootFinder hasConverged
		] whileTrue: [ roots add: x. 
					   pol := pol deflatedAt: x. 
					   pol degree > 0
						 ifFalse: [ ^roots].
					 ].
	^roots! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) - ( self at: n)])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesNumber: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^self class coefficients: ( coefficients reverse collect: [ :each | each * aNumber])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	| productCoefficients degree|
	degree := aPolynomial degree + self degree.
	productCoefficients := ( degree to: 0 by: -1)
			collect:[ :n | | sum |
					  sum := 0.
					  0 to: (degree - n)
						do: [ :k | sum := (self at: k) * (aPolynomial at: ( degree - n - k)) + sum].
					  sum
					].
	^self class coefficients: productCoefficients! !

!DhbPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answer the value of the polynomial for the specified variable value.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^coefficients inject: 0 into: [ :sum :each | sum * aNumber + each]! !

Object subclass: #DhbPolynomialLeastSquareFit
	instanceVariableNames: 'pointCollection degreePlusOne'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"Create a new instance of the receiver with given degree.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger! !

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger on: aCollectionOfPoints
		"Create a new instance of the receiver with given degree and points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger on: aCollectionOfPoints! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint into: aVectorOfVectors and: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| t p powers |
	p := 1.0.
	powers := aVector collect: [ :each | t := p. p := p * aWeightedPoint xValue. t].
	aVector accumulate: powers * ( aWeightedPoint yValue * aWeightedPoint weight).
	1 to: aVector size do:
		[ :k |
		  ( aVectorOfVectors at: k) accumulate: powers * ( ( powers at: k) * aWeightedPoint weight).
		].! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aWeightedPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^pointCollection add: aWeightedPoint! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeEquations
		"Private - Answer a pair Matrix/Vector defining the system of equations
		 to solve the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| rows vector |
	vector := ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself.
	rows := ( 1 to: degreePlusOne) collect: [ :k | ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself].
	pointCollection do:
		[ :each | self accumulate: each into: rows and: vector].
	^Array with: ( DhbSymmetricMatrix rows: rows) with: vector! !

!DhbPolynomialLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluate
		"Perform the least square fit and answers the fitted polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| system errorMatrix |
	system := self computeEquations.
	errorMatrix := ( system at: 1) inverse.
	^( DhbEstimatedPolynomial coefficients: errorMatrix * (system at: 2))
			errorMatrix: errorMatrix;
			yourself! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^self initialize: anInteger on: OrderedCollection new! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger on: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	pointCollection := aCollectionOfPoints.
	degreePlusOne := anInteger + 1.
	^self! !

Object subclass: #DhbProbabilityDensity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbProbabilityDensity subclass: #DhbAsymptoticChiSquareDistribution
	instanceVariableNames: 'degreeOfFreedom reducedDOF normalDistribution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbAsymptoticChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^super new initialize: anInteger! !

!DhbAsymptoticChiSquareDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^'Chi square distribution'! !

!DhbAsymptoticChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	normalDistribution changeParametersBy: aVector.! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a chi square value
		 distributed according to the receiver larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^( 1 - ( self distributionValue: aNumber)) *100! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| x |
	^aNumber > 0
		ifTrue: [ x := (aNumber * 2) sqrt.
				  ( DhbErfApproximation new value: (x - reducedDOF))
				]
		ifFalse:[ 0]! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	degreeOfFreedom := anInteger.
	reducedDOF := ( degreeOfFreedom * 2 - 1) sqrt.
	^self! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^12 / degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"Returns an Array containing the parameters of the distribution.
		 It is used to print out the distribution and for fitting.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: degreeOfFreedom! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbNormalDistribution random + reducedDOF) squared / 2! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	^( 2 / degreeOfFreedom) sqrt * 2! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| x |
	^aNumber > 0
		ifTrue: [ x := (aNumber * 2) sqrt.
				  ( DhbErfApproximation new normal: (x - reducedDOF)) / x
				]
		ifFalse:[ 0]! !

!DhbAsymptoticChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^degreeOfFreedom * 2! !

DhbProbabilityDensity subclass: #DhbBetaDistribution
	instanceVariableNames: 'alpha1 alpha2 gamma1 gamma2 logNorm incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBetaDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Beta distribution'! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the 
	given histogram using best guesses. This method can be used to 
	find the initial values for a fit. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 12/3/99"

	| average variance a b |
	(aHistogram minimum < 0 or: [aHistogram maximum > 1])
		ifTrue: [^nil].
	average := aHistogram average.
	variance := aHistogram variance.
	a := 1 - average / variance - 1 * average.
	a > 0 ifFalse: [^nil].
	b := 1 / average - 1 * a.
	b > 0 ifFalse: [^nil].
	^self shape: a shape: b! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbBetaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 shape: aNumber2 
	"Create an instance of the receiver with given shape parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 shape: aNumber2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha1 / ( alpha1 + alpha2)! !

!DhbBetaDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha1 := alpha1 + ( aVector at: 1).
	alpha2 := alpha2 + ( aVector at: 2).
	self computeNorm.
	gamma1 := nil.
	gamma2 := nil.
	incompleteBetaFunction := nil.! !

!DhbBetaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	logNorm := (alpha1 + alpha2) logGamma - alpha1 logGamma - alpha2 logGamma.! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	incompleteBetaFunction isNil
		ifTrue: [ incompleteBetaFunction := DhbIncompleteBetaFunction shape: alpha1 shape: alpha2].
	^incompleteBetaFunction value: aNumber! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
firstGammaDistribution
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	gamma1 isNil
		ifTrue: [ gamma1 := DhbGammaDistribution shape: alpha1 scale: 1].
	 ^gamma1! !

!DhbBetaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	(aNumber1 > 0 and: [aNumber2 > 0]) 
		ifFalse: [self error: 'Illegal distribution parameters'].
	alpha1 := aNumber1.
	alpha2 := aNumber2.
	self computeNorm.
	^self! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^3 * ( alpha1 + alpha2 + 1) * ( (alpha1 + alpha2) squared * 2 + ( ( alpha1 + alpha2 - 6) * alpha1 * alpha2)
			/ ( ( alpha1 + alpha2 + 2) * ( alpha1 + alpha2 + 3) * alpha1 * alpha2)) - 3! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha1 with: alpha2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| r |
	r := self firstGammaDistribution random.
	^r / ( self secondGammaDistribution random + r)! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
secondGammaDistribution
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	gamma2 isNil
		ifTrue: [ gamma2 := DhbGammaDistribution shape: alpha2 scale: 1].
	 ^gamma2! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^( alpha1 + alpha2 + 1) sqrt * 2 * ( alpha2 - alpha1) / ( ( alpha1 * alpha2) sqrt * ( alpha1 + alpha2 + 2))! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber 
	"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^(aNumber > 0 and: [ aNumber < 1]) 
		ifTrue: 
			[( ( aNumber ln * (alpha1 - 1) ) + ( ( 1 - aNumber) ln * ( alpha2 - 1)) + logNorm) exp]
		ifFalse: [0]! !

!DhbBetaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha1 * alpha2 / ( ( alpha1 + alpha2) squared * ( alpha1 + alpha2 + 1))! !

DhbProbabilityDensity subclass: #DhbCauchyDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCauchyDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Cauchy distribution'! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average
		scale: 4 * aHistogram variance 
				/ (Float pi * (aHistogram maximum squared + aHistogram minimum squared)) 
						sqrt! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create an instance of the receiver with center 0 and scale 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given center and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^mu! !

!DhbCauchyDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(( aNumber - mu) / beta) arcTan / Float pi + (1 / 2)! !

!DhbCauchyDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^( ( aNumber - (1 / 2)) * Float pi) tan * beta + mu! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"The standard deviation of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^beta / ( Float pi * ( beta squared + ( aNumber - mu) squared))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp denominator |
	dp := self value: aNumber.
	denominator := 1 / ( ( aNumber - mu) squared + beta squared).
	^Array with: dp
		   with: ( DhbVector with: 2 * dp * ( aNumber - mu) * denominator
							 with: dp * ( 1 / beta - ( 2 * beta * denominator)))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"The variance of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

DhbProbabilityDensity subclass: #DhbExponentialDistribution
	instanceVariableNames: 'beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbExponentialDistribution class methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:33'!
distributionName
	^ 'Exponential distribution'! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit."
	| mu |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	mu := aHistogram average.
	^mu > 0 ifTrue: [ self scale: aHistogram average]
			ifFalse:[ nil]! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
new
		"Create a new instance of the receiver with scale parameter 1."
	^super new initialize: 1! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
scale: aNumber
		"Create a new instance of the receiver with given scale parameter."
	^super new initialize: aNumber! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
average
		"Answer the average of the receiver."
	^beta! !

!DhbExponentialDistribution methodsFor: 'transformation' stamp: 'SergeStinckwich 3/8/2015 21:31'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector."
	beta := beta + ( aVector at: 1).! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber."

	^[1 - ( ( aNumber / beta negated) exp)]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'initialization' stamp: 'SergeStinckwich 3/8/2015 21:31'!
initialize: aNumber
		"Private - Set the scale parameter of the receiver to aNumber."
	aNumber > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	beta := aNumber.
	^self! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
kurtosis
		"Answer the kurtosis of the receiver."
	^6! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
parameters
	^ Array with: beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber."
	^(1 - aNumber) ln negated * beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
random
		"Answer a random number distributed according to the receiver."
	^DhbMitchellMooreGenerator new floatValue ln * beta negated! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
skewness
		"Answer the skewness of the receiver."
	^2! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
standardDeviation
		"Answer the standard deviation of the receiver."
	^beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval)."

	^[ ( aNumber / beta) negated exp / beta]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber."
	| dp |
	dp := self value: aNumber.
	^Array with: dp
		   with: ( DhbVector with: ( aNumber / beta - 1) * dp / beta)! !

DhbProbabilityDensity subclass: #DhbFisherSnedecorDistribution
	instanceVariableNames: 'dof1 dof2 norm chiSquareDistribution1 chiSquareDistribution2 incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom: anInteger1 degreeOfFreedom: anInteger2 
	"Create a new instance of the receiver with given degrees of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^super new initialize: anInteger1 and: anInteger2! !

!DhbFisherSnedecorDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Fisher-Snedecor distribution'! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "

	| n1 n2 a |
	aHistogram minimum < 0 ifTrue: [^nil].
	n2 := (2 / (1 - (1 / aHistogram average))) rounded.
	n2 > 0 ifFalse: [^nil].
	a := (n2 - 2) * (n2 - 4) * aHistogram variance / (n2 squared * 2).
	n1 := (0.7 * (n2 - 2) / (1 - a)) rounded.
	^n1 > 0 
		ifTrue: [self degreeOfFreedom: n1 degreeOfFreedom: n2]
		ifFalse: [nil]! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbFisherSnedecorDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
test: aStatisticalMoment1 with: aStatisticalMoment2 
	"Perform a consistency Fisher test (or F-test) on the variances of two statistical moments ( or histograms).
		 Answer the probability of passing the test.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^(self class degreeOfFreedom: aStatisticalMoment1 count
		degreeOfFreedom: aStatisticalMoment2 count) 
			distributionValue: aStatisticalMoment1 variance 
					/ aStatisticalMoment2 variance! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 Undefined if dof2 is smaller than 3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^dof2 > 2
		ifTrue: [ dof2 / ( dof2 - 2)]
		ifFalse:[ nil]! !

!DhbFisherSnedecorDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	dof1 := ( dof1 + ( aVector at: 1)) max: 1.
	dof2 := ( dof2 + ( aVector at: 2)) max: 1.
	self computeNorm.
	chiSquareDistribution1 := nil.
	chiSquareDistribution2 := nil.
	incompleteBetaFunction := nil.! !

!DhbFisherSnedecorDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	norm := ( dof1 ln * ( dof1 / 2) ) + ( dof2 ln * ( dof2 / 2) )
						- ( ( dof1 / 2) logBeta: ( dof2 / 2) ).! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a value
		 distributed according to the receiver outside of the
		 interval [ 1/aNumber, aNumber].
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	aNumber < 0
		ifTrue: [ self error: 'Confidence level argument must be positive'].
	^( 1- ( self acceptanceBetween: aNumber reciprocal and: aNumber)) * 100! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^1 - ( self incompleteBetaFunction value: ( dof2 / ( aNumber * dof1 + dof2)))! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
incompleteBetaFunction
	"Private - Answers the incomplete beta function used to compute
		 the symmetric acceptance integral of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "

	incompleteBetaFunction isNil 
		ifTrue: 
			[incompleteBetaFunction := DhbIncompleteBetaFunction shape: dof2 / 2
						shape: dof1 / 2].
	^incompleteBetaFunction! !

!DhbFisherSnedecorDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger1 and: anInteger2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	dof1 := anInteger1.
	dof2 := anInteger2.
	self computeNorm.
	^self! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: dof1 with: dof2! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	chiSquareDistribution1 isNil
		ifTrue: [ chiSquareDistribution1 := DhbChiSquareDistribution degreeOfFreedom: dof1.
				  chiSquareDistribution2 := DhbChiSquareDistribution degreeOfFreedom: dof2.
				].
	^chiSquareDistribution1 random * dof2 / ( chiSquareDistribution2 random * dof1)! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( norm + ( aNumber ln * ( dof1 / 2 - 1) ) - ( (aNumber * dof1 + dof2) ln * ( ( dof1 + dof2) / 2))) exp]
		ifFalse:[ 0]! !

!DhbFisherSnedecorDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 Undefined if dof2 is smaller than 5.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^dof2 > 4 ifTrue: [ dof2 squared * 2 * ( dof1 + dof2 - 2) / ( ( dof2 - 2) squared * dof1 * ( dof2 - 4))]
				   ifFalse:[ nil]! !

DhbProbabilityDensity subclass: #DhbFisherTippettDistribution
	instanceVariableNames: 'alpha beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFisherTippettDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Fisher-Tippett distribution'! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	| beta |
	beta := aHistogram standardDeviation.
	beta = 0 ifTrue: [^nil].
	beta := beta * (6 sqrt / Float pi).
	^self shape: aHistogram average - (0.5772156649 * beta) scale: beta! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create a standard version of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^0.577256649 * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg < DhbFloatingPointMachine new largestExponentArgument negated
					ifTrue: [ ^0]
					ifFalse:[arg negated exp].
	^arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ 1]
						ifFalse:[ arg negated exp]! !

!DhbFisherTippettDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	aNumber2 > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	^self! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:01'!
integralFrom: aNumber1 to: aNumber2
		"Private - Compute the integral of the receiver from aNumber1 to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator function: [:x|self value:x] from: aNumber1 to: aNumber2) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 10:56'!
integralUpTo: aNumber
		"Private - Compute the integral of the receiver from -infinity to aNumber.
		 aNumber must be below 0 (no checking!!).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator function:
			[ :x | x = 0 ifTrue: [ 0] ifFalse: [ ( self value: 1 / x) / x squared] ]
			from: 1 / aNumber to: 0) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^2.4! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/4/99 "
	| t |
	[ t := DhbMitchellMooreGenerator new floatValue ln negated.
	  t > 0] whileFalse: [].
	^t ln negated * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^1.3! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Float pi * beta / ( 6 sqrt)! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ ^0]
						ifFalse:[arg negated exp + arg].
	^arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ 0]
						ifFalse:[ arg negated exp / beta]! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp dy y|
	dp := self value: aNumber.
	y := ( aNumber - alpha) / beta.
	dy := ( y negated exp - 1).
	^Array with: dp
		   with: ( DhbVector with: dy * dp / beta negated
							 with: dp * ( y * dy + 1) / beta negated)! !

DhbProbabilityDensity subclass: #DhbGammaDistribution
	instanceVariableNames: 'alpha beta norm randomCoefficients incompleteGammaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbGammaDistribution subclass: #DhbChiSquareDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom: anInteger
		"Create a new instance of the receiver with given degree of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^anInteger > 40
		ifTrue: [ DhbAsymptoticChiSquareDistribution degreeOfFreedom: anInteger]
		ifFalse:[ super shape: anInteger / 2 scale: 2]! !

!DhbChiSquareDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Chi square distribution'! !

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| dof |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	dof := aHistogram average rounded.
	^dof > 0 ifTrue: [ self degreeOfFreedom: aHistogram average rounded]
			 ifFalse:[ nil]! !

!DhbChiSquareDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2
		"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbChiSquareDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 13/4/99 "
	super changeParametersBy: (Array with: aVector first / 2 with: 0).! !

!DhbChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a chi square value
		 distributed according to the receiver larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	^( 1 - ( self distributionValue: aNumber)) *100! !

!DhbChiSquareDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha * 2! !

!DhbGammaDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Gamma distribution'! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| alpha beta |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	alpha := aHistogram average.
	beta := aHistogram variance / alpha.
	^[ self shape: alpha / beta scale: beta] on: Error do: [ :signal | signal return: nil]! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbGammaDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2
		"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 scale: aNumber2! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^alpha * beta! !

!DhbGammaDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).
	self computeNorm.
	incompleteGammaFunction := nil.
	randomCoefficients := nil.! !

!DhbGammaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	norm := beta ln * alpha + alpha logGamma.! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self incompleteGammaFunction value: aNumber / beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
incompleteGammaFunction
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	incompleteGammaFunction isNil 
		ifTrue: 
			[incompleteGammaFunction := DhbIncompleteGammaFunction shape: alpha].
	^incompleteGammaFunction! !

!DhbGammaDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber1 > 0 and: [ aNumber2 > 0])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	self computeNorm.
	^self! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeRandomCoefficientsForLargeAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| a b q d |
	a := 1 / ( 2 * alpha - 1) sqrt.
	b := alpha - (4 ln).
	q := 1 / a + alpha.
	d := 4.5 ln + 1.
	^Array with: a with: b with: q with: d! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeRandomCoefficientsForSmallAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| e |
	e := 1 exp.
	^( e + alpha) / e! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^6 / alpha! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( alpha > 1 ifTrue: [ self randomForLargeAlpha]
						ifFalse:[ self randomForSmallAlpha]) * beta! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomCoefficientsForLargeAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	randomCoefficients isNil
		ifTrue: [ randomCoefficients := self initializeRandomCoefficientsForLargeAlpha].
	^randomCoefficients! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomCoefficientsForSmallAlpha
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	randomCoefficients isNil
		ifTrue: [ randomCoefficients := self initializeRandomCoefficientsForSmallAlpha].
	^randomCoefficients! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomForLargeAlpha
		"Private - Generate a random number distributed according to the receiver
						when alpha > 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	[ true] whileTrue: [
	| u1 u2 c v y z w|
	u1 := DhbMitchellMooreGenerator new floatValue.
	u2 := DhbMitchellMooreGenerator new floatValue.
	c := self randomCoefficientsForLargeAlpha.
	v := ( u1 / ( 1 - u1)) ln * (c at: 1).
	y := v exp * alpha.
	z := u1 squared * u2.
	w := ( c at: 3) * v + ( c at: 2) - y.
	( c at: 4) + w >= ( 4.5 * z) ifTrue: [ ^y].
	z ln <= w ifTrue: [ ^y].
								].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomForSmallAlpha
		"Private - Generate a random number distributed according to the receiver
						when alpha < 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	[ true] whileTrue: [
	| p |
	p := DhbMitchellMooreGenerator new floatValue * self randomCoefficientsForSmallAlpha.
	p > 1
		ifTrue: [ | y |
					 y := ( ( self randomCoefficientsForSmallAlpha - p) / alpha) ln negated.
					 DhbMitchellMooreGenerator new floatValue <= ( y raisedTo: ( alpha - 1))
					 	ifTrue: [ ^y].
					]
		ifFalse: [ | y |
						y := p raisedTo: ( 1 / alpha).
					 DhbMitchellMooreGenerator new floatValue <= ( y negated exp)
					 	ifTrue: [ ^y].
					 ].
								].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^2 / alpha sqrt! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( aNumber ln * (alpha - 1) - (aNumber / beta) - norm) exp]
		ifFalse:[ 0].! !

!DhbGammaDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta squared * alpha! !

DhbProbabilityDensity subclass: #DhbHistogrammedDistribution
	instanceVariableNames: 'histogram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHistogrammedDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Experimental distribution'! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram
		"Create a new instance of the receiver corresponding to a histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^super new initialize: aHistogram! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( histogram countsBetween: ( aNumber1 max: histogram minimum)
						and: ( aNumber2 min: histogram maximum) ) / histogram totalCount! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram average! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber < histogram minimum
		ifTrue: [ 0]
		ifFalse:[ aNumber < histogram maximum
							ifTrue: [ ( histogram countsUpTo: aNumber) / histogram totalCount]
							ifFalse:[ 1]
					]! !

!DhbHistogrammedDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aHistogram
		"Private - Defines the histogram of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	aHistogram count = 0
		ifTrue: [ self error: 'Cannot define probability density on an empty histogram'].
	histogram := aHistogram.
	^self! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram kurtosis! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'WernerKassens 4/9/2014 14:24'!
parameters 
^nil! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose distribution is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram inverseDistributionValue: aNumber! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram skewness! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram standardDeviation! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^( aNumber >= histogram minimum and: [ aNumber < histogram maximum])
		ifTrue: [ ( histogram countAt: aNumber) / ( histogram totalCount * histogram binWidth)]
		ifFalse:[ 0]! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram variance! !

DhbProbabilityDensity subclass: #DhbLaplaceDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLaplaceDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Laplace distribution'! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average scale: (aHistogram variance / 2) sqrt! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^self shape: 0 scale: 1! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^mu! !

!DhbLaplaceDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > mu
		ifTrue: [ 1 - ( ( ( aNumber - mu) / beta) negated exp / 2)]
		ifFalse:[ ( ( ( aNumber - mu) / beta) exp / 2)]! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^3! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| r |
	r := DhbMitchellMooreGenerator new floatValue ln * beta negated.
	^DhbMitchellMooreGenerator new floatValue > 0.5
		ifTrue: [ mu + r]
		ifFalse:[ mu - r]! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta * ( 2 sqrt)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^( ( aNumber - mu) / beta) abs negated exp / ( 2 * beta)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp |
	dp := self value: aNumber.
	^Array	with: dp
			with: ( DhbVector with: ( aNumber - mu) sign * dp / beta
							  with: ( ( ( aNumber - mu) abs / beta - 1) * dp / beta))! !

DhbProbabilityDensity subclass: #DhbNormalDistribution
	instanceVariableNames: 'mu sigma'
	classVariableNames: 'NextRandom'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Normal distribution'! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	^self new: aHistogram average sigma: aHistogram standardDeviation! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with mu=0 and sigma=1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self new: 0 sigma: 1! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to a (0,1) normal distribution.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| v1 v2 w y |
	NextRandom isNil
		ifTrue: [ [ v1 := Number random * 2 - 1.
					v2 := Number random * 2 - 1.
					w := v1 squared + v2 squared.
					w > 1 ] whileTrue: [].
				  y := ( ( w ln * 2 negated) / w) sqrt.
				v1 := y * v1.
				NextRandom := y * v2.
				]
		ifFalse:[ v1 :=NextRandom.
				  NextRandom := nil.
				].
	^v1! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^mu! !

!DhbNormalDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	mu := mu + ( aVector at: 1).
	sigma := sigma + ( aVector at: 2).! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^DhbErfApproximation new value: ( ( aNumber - mu) / sigma)! !

!DhbNormalDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 sigma: aNumber2
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	mu := aNumber1.
	sigma := aNumber2.
	^self! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self class random * sigma + mu! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbErfApproximation new normal: (aNumber - mu) / sigma) / sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| dp y |
	y := ( aNumber - mu) / sigma.
	dp := ( DhbErfApproximation new normal: y) / sigma.
	^Array with: dp
		   with: ( DhbVector with: dp * y / sigma
							 with: dp * ( y squared - 1) / sigma)! !

!DhbProbabilityDensity class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Unknown distribution'! !

!DhbProbabilityDensity class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( self distributionValue: aNumber2) - ( self distributionValue: aNumber1)! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
approximatedValueAndGradient: aNumber
		"Private - gradients an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 The gradient is computed by approximation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| delta parameters dp gradient n |
	parameters := self parameters.
	n := parameters size.
	dp := self value: aNumber.
	delta := Array new: n.
	delta atAllPut: 0.
	gradient := DhbVector new: n.
	1 to: n do:
		[ :k |
		  delta at: k put: ( parameters at: k) * 0.0001.
		  self changeParametersBy: delta.
		  gradient at: k put: ( ( ( self value: aNumber) - dp) / ( delta at: k)).
		  delta at: k put: ( delta at: k ) negated.
		  k > 1
			ifTrue: [ delta at: ( k - 1) put: 0].
		].
	self changeParametersBy: delta.
	^Array with: dp with: gradient! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self subclassResponsibility.! !

!DhbProbabilityDensity methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector 
	self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionFunction
		" (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^DhbProbabilityDistributionFunction density: self! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverseDistributionValue: aNumber
		"Answer the number whose distribution value is aNumber.
		 NOTE: Subclass MUST NOT overwrite this method.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( aNumber between: 0 and: 1)
			ifTrue: [ self privateInverseDistributionValue: aNumber]
			ifFalse:[ self error: 'Illegal argument for inverse distribution value']! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
	"Returns an Array containing the parameters of the distribution. 
	It is used to print out the distribution and for fitting. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/3/99"

	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 4/3/99"

	| params |
	aStream nextPutAll: self class distributionName.
	(params := self parameters) notNil
		ifTrue: 
			[| first |
			first := true.
			aStream nextPut: $(.
			params
				do: 
					[:each | 
					first
						ifTrue: [first := false]
						ifFalse: [aStream nextPut: $,].
					aStream space.
					each printOn: aStream].
			aStream nextPut: $)]! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose distribution is aNumber.
		 NOTE: Subclass may overwrite this method for faster computation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbNewtonZeroFinder function: [ :x | ( self distributionValue: x) - aNumber] derivative: self)
		initialValue: self average / (1 - aNumber); evaluate! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self privateInverseDistributionValue: DhbMitchellMooreGenerator new floatValue! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self variance sqrt! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self subclassResponsibility.! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^self approximatedValueAndGradient: aNumber! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self standardDeviation squared! !

DhbProbabilityDensity subclass: #DhbProbabilityDensityWithUnknownDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbProbabilityDensityWithUnknownDistribution subclass: #DhbLogNormalDistribution
	instanceVariableNames: 'normalDistribution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLogNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Log normal distribution'! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average variance sigma2 |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	average > 0
		ifFalse: [ ^nil].
	variance := aHistogram variance.
	sigma2 := ( variance / average squared + 1) ln.
	sigma2 > 0
		ifFalse: [ ^nil].
	^self new: ( average ln - (sigma2 * 0.5)) sigma: sigma2 sqrt! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with mu=0 and sigma=1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self new: 0 sigma: 1! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution variance*0.5 + normalDistribution average) exp! !

!DhbLogNormalDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	normalDistribution changeParametersBy: aVector.! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fourthCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared squared) * ( x squared)
		* ( ( ( x squared * x - 4) * ( x squared) + 6) * x - 3)! !

!DhbLogNormalDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 sigma: aNumber2
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	normalDistribution := DhbNormalDistribution new: aNumber1 sigma: aNumber2.
	^self! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^( ( x + 2) * x + 3) * ( x squared) - 6! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution parameters! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution random exp! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^(x - 1) sqrt * (x + 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
thirdCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared * y) * ( x raisedTo: (3/2))
		* ( ( x squared negated + 3) * x - 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( normalDistribution value: aNumber ln) / aNumber]
		ifFalse:[ 0]! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution average * 2 + normalDistribution variance) exp * ( normalDistribution variance exp - 1)! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:39'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator function: [:x| self value:x] from: aNumber1 to: aNumber2) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:38'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 This general purpose routine uses numerical integration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator function: [:x| self value:x] from: self lowestValue to: aNumber) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
lowestValue
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "
	^0! !

DhbProbabilityDensity subclass: #DhbStudentDistribution
	instanceVariableNames: 'degreeOfFreedom norm chiSquareDistribution incompleteBetaFunction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asymptoticLimit
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/4/99 "
	^30! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom: anInteger 
	"Create a new instance of the receiver with anInteger degrees of freedom.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "

	^anInteger > self asymptoticLimit 
		ifTrue: [DhbNormalDistribution new]
		ifFalse: 
			[anInteger = 1 
				ifTrue: [DhbCauchyDistribution shape: 0 scale: 1]
				ifFalse: [super new initialize: anInteger]]! !

!DhbStudentDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Student distribution'! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 5/3/99 "
	| dof var |
	var := aHistogram variance.
	var = 0
		ifTrue: [ ^nil].
	dof :=  ( 2 / (1 - (1 / aHistogram variance))) rounded max: 1.
	^dof > self asymptoticLimit ifTrue: [ nil]
								ifFalse:[ self degreeOfFreedom: dof]! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbStudentDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
test: aStatisticalMoment1 with: aStatisticalMoment2
		"Preform a consistency Student test (or t-test) on the averages of  two statistical moments ( or histograms).
		 Answers the probability of failing the test.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	| t |
	t := ( aStatisticalMoment1 average - aStatisticalMoment2 average) abs.
	^1 - ( ( self class degreeOfFreedom: (  aStatisticalMoment1 count + aStatisticalMoment2 count - 2)) acceptanceBetween: t negated and: t)! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^0! !

!DhbStudentDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	degreeOfFreedom := degreeOfFreedom + ( aVector at: 1).
	self computeNorm.! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chiSquareDistribution
		"Private - Answer the chi square distribution used to generate
		 random numbers for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	chiSquareDistribution isNil
		ifTrue: [ chiSquareDistribution := DhbChiSquareDistribution degreeOfFreedom: (degreeOfFreedom - 1)].
	^chiSquareDistribution! !

!DhbStudentDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	norm := ( ( degreeOfFreedom / 2 logBeta: ( 1 / 2) ) + ( degreeOfFreedom ln / 2)) negated.! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel: aNumber
		"Answer the probability in percent of finding a value
		 distributed according to the receiver with an absolute value
		 larger than aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^( 1 - ( self symmetricAcceptance: aNumber abs)) * 100! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	aNumber = 0
		ifTrue: [ ^1/2].
	^( aNumber > 0
		ifTrue: [ 2 - ( self symmetricAcceptance: aNumber abs)]
		ifFalse:[ self symmetricAcceptance: aNumber abs]) / 2! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
incompleteBetaFunction
	"Private - Answers the incomplete beta function used to compute
		 the symmetric acceptance integral of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "

	incompleteBetaFunction isNil 
		ifTrue: 
			[incompleteBetaFunction := DhbIncompleteBetaFunction 
						shape: degreeOfFreedom / 2
						shape: 0.5].
	^incompleteBetaFunction! !

!DhbStudentDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	anInteger > 0
		ifFalse: [ self error: 'Degree of freedom must be positive'].
	degreeOfFreedom := anInteger.
	self computeNorm.
	^self! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined if the degree of freedom is less than 5.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^degreeOfFreedom > 4 ifTrue: [ 6 / ( degreeOfFreedom - 4)]
						 ifFalse:[ nil]! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: degreeOfFreedom! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^DhbNormalDistribution random * ( ( (degreeOfFreedom - 1) / self chiSquareDistribution random ) sqrt)! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
symmetricAcceptance: aNumber
		"Private - Compute the acceptance of the receiver between -aNumber and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/3/99 "
	^ self incompleteBetaFunction value: ( degreeOfFreedom / ( aNumber squared + degreeOfFreedom))! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( norm - ( ( aNumber squared / degreeOfFreedom + 1) ln * ( ( degreeOfFreedom + 1) / 2))) exp! !

!DhbStudentDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 Undefined if the degree of freedom is less than 3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^degreeOfFreedom > 2 ifTrue: [ degreeOfFreedom / ( degreeOfFreedom - 2)]
						 ifFalse:[ nil]! !

DhbProbabilityDensity subclass: #DhbTriangularDistribution
	instanceVariableNames: 'lowLimit highLimit peak'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbTriangularDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Triangular distribution'! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self new: c from: ( c - b) to: ( c + b).! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create an instance of the receiver with peak at 1/2 and limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^self new: (1 / 2) from: 0 to: 1! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber1 from: aNumber2 to: aNumber3
		"Create an instance of the receiver with peak at aNumber1 and limits aNumber2 and aNumber3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 from: aNumber2 to: aNumber3! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit + peak + highLimit) / 3! !

!DhbTriangularDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).
	peak := peak + ( aVector at: 3).! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
		"changed last line so that 1 is returned, if aNumber >highLimit "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 ( aNumber - lowLimit) squared / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 1 - ( ( highLimit - aNumber) squared / norm)
													]
										ifFalse:[ ( peak - lowLimit) / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ aNumber >lowLimit ifTrue: [1] ifFalse: [0]]! !

!DhbTriangularDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 from: aNumber2 to: aNumber3
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber2 < aNumber3 and: [ aNumber1 between: aNumber2 and: aNumber3])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	peak := aNumber1.
	lowLimit := aNumber2.
	highLimit := aNumber3.
	^self! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverseAcceptanceAfterPeak: aNumber
		"Private - Compute inverse acceptance function in the region after the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ highLimit - ( ( ( 1 - aNumber) * ( highLimit - lowLimit) * ( highLimit - peak)) sqrt)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverseAcceptanceBeforePeak: aNumber
		"Private - Compute inverse acceptance function in the region before the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ ( aNumber * ( highLimit - lowLimit) * ( peak - lowLimit)) sqrt + lowLimit! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(-6/10)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit with: peak! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( peak - lowLimit) / ( highLimit - lowLimit) > aNumber
			ifTrue: [ self inverseAcceptanceBeforePeak: aNumber]
			ifFalse: [ self inverseAcceptanceAfterPeak: aNumber]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(((lowLimit squared * lowLimit + ( peak squared * peak) + ( highLimit squared * highLimit) ) / 135)
	-(((lowLimit squared * peak) + (lowLimit squared * highLimit) + (peak squared * lowLimit) + (peak squared * highLimit) + (highLimit squared * lowLimit) + (highLimit squared * peak))/90)
	+( 2 * lowLimit * peak * highLimit / 45)) / ( self standardDeviation raisedToInteger: 3)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 2 * ( aNumber - lowLimit) / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 2 * ( highLimit - aNumber) / norm
													]
										ifFalse:[ 2 / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ 0]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit squared + peak squared + highLimit squared - ( lowLimit * peak) - ( lowLimit * highLimit) - ( peak * highLimit)) / 18! !

DhbProbabilityDensity subclass: #DhbUniformDistribution
	instanceVariableNames: 'lowLimit highLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Uniform distribution'! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
from: aNumber1 to: aNumber2
		"Create a new instance of the receiver with given limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 to: aNumber2! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self from: ( c - b) to: ( c + b).! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self from: 0 to: 1! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit + lowLimit) / 2! !

!DhbUniformDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber < lowLimit
		ifTrue: [ ^0].
	^aNumber < highLimit
		ifTrue: [ (aNumber - lowLimit) / ( highLimit - lowLimit)]
		ifFalse:[ 1]! !

!DhbUniformDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 to: aNumber2
		"Private - Defines the limits of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber1 < aNumber2
		ifFalse: [ self error: 'Illegal distribution parameters'].
	lowLimit := aNumber1.
	highLimit := aNumber2.
	^self! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^-12 / 10! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(highLimit - lowLimit) * aNumber + lowLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) / 3.46410161513775 "12 sqrt"! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ 1/( highLimit - lowLimit)]
		ifFalse:[ 0]! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) squared / 12! !

DhbProbabilityDensity subclass: #DhbWeibullDistribution
	instanceVariableNames: 'alpha beta norm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbWeibullDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Weibull distribution'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average xMin xMax accMin accMax |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	xMin := ( aHistogram minimum + average) / 2.
	accMin := ( aHistogram countsUpTo: xMin) / aHistogram totalCount.
	xMax := ( aHistogram maximum + average) / 2.
	accMax := ( aHistogram countsUpTo: xMax) / aHistogram totalCount.
	^[self solve: xMin acc: accMin upper: xMax acc: accMax]
			on: Error do: [ :signal | signal return: nil]! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
solve: lowX acc: lowAcc upper: highX acc: highAcc 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| lowLnAcc highLnAcc deltaLnAcc lowLnX highLnX |
	lowLnAcc := (1 - lowAcc) ln negated ln.
	highLnAcc := (1 - highAcc) ln negated ln.
	deltaLnAcc := highLnAcc - lowLnAcc.
	lowLnX := lowX ln.
	highLnX := highX ln.
	^self shape: deltaLnAcc / (highLnX - lowLnX)
		scale: ((highLnAcc * lowLnX - (lowLnAcc * highLnX)) / deltaLnAcc) exp! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(1 / alpha) gamma * beta / alpha! !

!DhbWeibullDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).
	self computeNorm.! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	norm := alpha/ ( beta raisedTo: alpha).! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ 1 - ( ( ( aNumber / beta) raisedTo: alpha) negated exp)]
		ifFalse:[ 0]! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber1 > 0 and: [ aNumber2 > 0])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	self computeNorm.
	^self! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( (1 - aNumber) ln negated raisedTo: ( 1 / alpha)) * beta! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( ( aNumber / beta) raisedTo: alpha) negated exp * ( aNumber raisedTo: ( alpha - 1)) * norm! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( beta squared / alpha) * ( (2 / alpha) gamma * 2 - ( (1 / alpha ) gamma squared / alpha))! !

Object subclass: #DhbProbabilityDistributionFunction
	instanceVariableNames: 'probabilityDensity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProbabilityDistributionFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
density: aProbabilityDensity
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self new initialize: aProbabilityDensity! !

!DhbProbabilityDistributionFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aProbabilityDensity
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	probabilityDensity := aProbabilityDensity.
	^self! !

!DhbProbabilityDistributionFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensity distributionValue: aNumber! !

Object subclass: #DhbProjectedOneVariableFunction
	instanceVariableNames: 'index function argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProjectedOneVariableFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aVectorFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initialize: aVectorFunction! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument at: index put: aNumber; yourself! !

!DhbProjectedOneVariableFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
bumpIndex
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1]
		ifFalse:[ index := index + 1.
				  index > argument size
					ifTrue: [ index := 1].
				].! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
index
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1].
	^index! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	function := aFunction.
	^self! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := anArrayOrVector copy.! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := anInteger.! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^function value: ( self argumentWith: aNumber)! !

DhbProjectedOneVariableFunction subclass: #DhbVectorProjectedFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^aNumber * self direction + self origin! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
direction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^index! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
direction: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self origin printOn: aStream.
	aStream nextPutAll: ' ('.
	self direction printOn: aStream.
	aStream nextPut: $).! !

Object subclass: #DhbScaledProbabilityDensityFunction
	instanceVariableNames: 'probabilityDensityFunction count binWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'WernerKassens 4/6/2014 13:05'!
histogram: aHistogram against: aProbabilityDensityFunction 
	"Create a new instance of the receiver with given probability density function and histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
		^self new 
		initialize: aProbabilityDensityFunction
		binWidth: aHistogram binWidth
		count: aHistogram totalCount! !

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"Create a new instance of the receiver with given probability density function and histogram. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 3/3/99"

	| dp |
	^(dp := aProbabilityDensityFunctionClass fromHistogram: aHistogram) isNil
		ifTrue: [nil]
		ifFalse: [self histogram: aHistogram against: dp]! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	count := count + aVector last.
	probabilityDensityFunction changeParametersBy: aVector.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionFunction
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensityFunction distributionFunction! !

!DhbScaledProbabilityDensityFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aProbabilityDensityFunction binWidth: aNumber count: anInteger 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	probabilityDensityFunction := aProbabilityDensityFunction.
	binWidth := aNumber.
	count := anInteger.
	^self! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	^probabilityDensityFunction parameters copyWith: count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	super printOn: aStream.
	aStream nextPut: $[;
			nextPutAll: probabilityDensityFunction class distributionName;
			nextPut: $].! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setCount: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	count := aNumber.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^(probabilityDensityFunction value: aNumber) * binWidth * count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber 
	"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	| g temp |
	g := probabilityDensityFunction valueAndGradient: aNumber.
	temp := binWidth * count.
	^Array with: g first * temp
		   with: ( (g last collect: [:each | each * temp]) copyWith: g first * binWidth)! !

Object subclass: #DhbSeriesTermServer
	instanceVariableNames: 'x lastTerm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbSeriesTermServer subclass: #DhbIncompleteBetaFractionTermServer
	instanceVariableNames: 'alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^1! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber1 second: aNumber2 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	| n n2 |
	n := anInteger // 2.
	n2 := 2 * n.
	^Array with: ( n2 < anInteger 
		ifTrue: 
			[x negated * (alpha1 + n) * (alpha1 + alpha2 + n) 
				/ ((alpha1 + n2) * (alpha1 + 1 + n2))]
		ifFalse: [x * n * (alpha2 - n) / ((alpha1 + n2) * (alpha1 - 1 + n2))])
			with: 1! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaFractionTermServer
	instanceVariableNames: 'alpha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	lastTerm := x - alpha + 1.
	^lastTerm! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := lastTerm + 2.
	^Array with: (alpha - anInteger) * anInteger with: lastTerm! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaSeriesTermServer
	instanceVariableNames: 'alpha sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := 1 / alpha.
	sum := alpha.
	^lastTerm! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termAt: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	sum := sum + 1.
	lastTerm := lastTerm * x / sum.
	^lastTerm! !

!DhbSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	x := aNumber asFloat.! !

Object subclass: #DhbStatisticalMoments
	instanceVariableNames: 'moments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbStatisticalMoments subclass: #DhbFastStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFastStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var |
	var := 1.
	1 to: moments size
		do: 
			[:n | 
			moments at: n put: (moments at: n) + var.
			var := var * aNumber]! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	self count = 0 ifTrue: [^nil].
	^(moments at: 2) / self count! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| var x1 x2 x3 x4 kFact kConst n m4 xSquared |
	n := self count.
	n < 4 ifTrue: [^nil].
	var := self variance.
	var = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	x4 := (moments at: 5) / n.
	xSquared := x1 squared.
	m4 := x4 - (4 * x1 * x3) + (6 * x2 * xSquared) - (xSquared squared * 3).
	kFact := n * (n + 1) / (n - 1) / (n - 2) / (n - 3).
	kConst := 3 * (n - 1) * (n - 1) / (n - 2) / (n - 3).
	^kFact * (m4 * n / var squared) - kConst! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| x1 x2 x3 n stdev |
	n := self count.
	n < 3 ifTrue: [^nil].
	stdev := self standardDeviation.
	stdev = 0 ifTrue: [^nil].
	x1 := (moments at: 2) / n.
	x2 := (moments at: 3) / n.
	x3 := (moments at: 4) / n.
	^(x3 - (3 * x1 * x2) + (2 * x1 * x1 * x1)) * n * n 
		/ (stdev squared * stdev * (n - 1) * (n - 2))! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
unnormalizedVariance
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "

	^(moments at: 3) - ((moments at: 2) squared * self count)! !

!DhbFastStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| n |
	n := self count.
	n < 2 ifTrue: [^nil].
	^((moments at: 3) - ((moments at: 2) squared / n)) / (n - 1)! !

DhbStatisticalMoments subclass: #DhbFixedStatisticalMoments
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^super new: 4! !

!DhbFixedStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbFixedStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/6/99 "

	| correction n n1 c2 c3 |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	c2 := correction squared.
	c3 := c2 * correction.
	moments
		at: 5
			put: ((moments at: 5) + ((moments at: 4) * correction * 4) 
					+ ((moments at: 3) * c2 * 6) + (c2 squared * (n squared * n + 1))) 
					* n / n1;
		at: 4
			put: ((moments at: 4) + ((moments at: 3) * correction * 3) 
					+ (c3 * (1 - n squared))) * n 
					/ n1;
		at: 3 put: ((moments at: 3) + (c2 * (1 + n))) * n / n1;
		at: 2 put: (moments at: 2) - correction;
		at: 1 put: n1! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^self new: 4! !

!DhbStatisticalMoments class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"anInteger is the degree of the highest central moments
		 accumulated within the created instance.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^super new initialize: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| correction n n1 oldSums pascal nTerm cTerm term |
	n := moments at: 1.
	n1 := n + 1.
	correction := ((moments at: 2) - aNumber) / n1.
	oldSums := moments copyFrom: 1 to: moments size.
	moments
		at: 1 put: n1;
		at: 2 put: (moments at: 2) - correction.
	pascal := Array new: moments size.
	pascal atAllPut: 0.
	pascal
		at: 1 put: 1;
		at: 2 put: 1.
	nTerm := -1.
	cTerm := correction.
	n1 := n / n1.
	n := n negated.
	3 to: moments size
		do: 
			[:k | 
			cTerm := cTerm * correction.
			nTerm := n * nTerm.
			term := cTerm * (1 + nTerm).
			k to: 3
				by: -1
				do: 
					[:l | 
					pascal at: l put: (pascal at: l - 1) + (pascal at: l).
					term := (pascal at: l) * (oldSums at: l) + term.
					oldSums at: l put: (oldSums at: l) * correction].
			pascal at: 2 put: (pascal at: 1) + (pascal at: 2).
			moments at: k put: term * n1]! !

!DhbStatisticalMoments methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asWeightedPoint: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^DhbWeightedPoint point: aNumber @ self average error: self errorOnAverage! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	self count = 0 ifTrue: [^nil].
	^moments at: 2! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
centralMoment: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/6/99 "

	^moments at: anInteger + 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
count
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/6/99 "

	^moments at: 1! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorOnAverage
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 02-Jan-00 "
	^( self variance / self count) sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fConfidenceLevel: aStatisticalMomentsOrHistogram 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| fValue |
	fValue := self variance/ aStatisticalMomentsOrHistogram variance.
	^fValue < 1
		ifTrue: [ (DhbFisherSnedecorDistribution degreeOfFreedom: aStatisticalMomentsOrHistogram count
						degreeOfFreedom: self count) 
										confidenceLevel: fValue reciprocal]
		ifFalse:[ (DhbFisherSnedecorDistribution degreeOfFreedom: self count
						degreeOfFreedom: aStatisticalMomentsOrHistogram count) 
										confidenceLevel: fValue]! !

!DhbStatisticalMoments methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger 
	"Private - ( anInteger - 1) is the degree of the highest accumulated central moment.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments := Array new: anInteger.
	self reset.
	^self! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n n1 n23 |
	n := self count.
	n < 4 ifTrue: [^nil].
	n23 := (n - 2) * (n - 3).
	n1 := n - 1.
	^((moments at: 5) * n squared * (n + 1) / (self variance squared * n1) 
		- (n1 squared * 3)) / n23! !

!DhbStatisticalMoments methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	moments atAllPut: 0! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "

	| n v |
	n := self count.
	n < 3 ifTrue: [^nil].
	v := self variance.
	^(moments at: 4) * n squared / ((n - 1) * (n - 2) * (v sqrt * v))! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	^self variance sqrt! !

!DhbStatisticalMoments methodsFor: 'testing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
tConfidenceLevel: aStatisticalMomentsOrHistogram
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	| sbar dof |
	dof := self count + aStatisticalMomentsOrHistogram count - 2.
	sbar := ( ( self unnormalizedVariance + aStatisticalMomentsOrHistogram unnormalizedVariance) / dof) sqrt.
	^( DhbStudentDistribution degreeOfFreedom: dof)
		confidenceLevel: ( self average - (aStatisticalMomentsOrHistogram average))
							/ ( ( 1 / self count + ( 1 / aStatisticalMomentsOrHistogram count)) sqrt * sbar)! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
unnormalizedVariance
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/22/00 "
	^( self centralMoment: 2) * self count! !

!DhbStatisticalMoments methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/5/99 "
	| n |
	n := self count.
	n < 2
		ifTrue: [ ^nil].
	^self unnormalizedVariance / ( n - 1)! !

Object subclass: #DhbVectorAccumulator
	instanceVariableNames: 'count average'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbVectorAccumulator subclass: #DhbCovarianceAccumulator
	instanceVariableNames: 'covariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: anArray
		"Accumulate anArray into the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| delta count1 r|
	count1 := count.
	delta := super accumulate: anArray.
	r := count1 / count.
	1 to: delta size
		do: [ :n |
			  1 to: n do:
				  [ :m |
					( covariance at: n) at: m put: ( count1 * ( delta at: n) * ( delta at: m) + ( r * ( ( covariance at: n) at: m))).
				  ].
			].! !

!DhbCovarianceAccumulator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
covarianceMatrix
		"Answer a matrix containing the covariance of the accumulated data.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| rows n |
	n := 0.
	rows := covariance collect:
				[ :row | n := n + 1. row, ( ( ( n + 1) to: covariance size) collect: [ :m | ( covariance at: m) at: n ])].
	^DhbSymmetricMatrix rows: rows! !

!DhbCovarianceAccumulator methodsFor: 'initialization' stamp: 'WernerKassens 4/15/2014 09:57'!
initialize: anInteger
		"Private - Initialize the receiver to accumulate vectors of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	covariance := ( ( 1 to: anInteger) collect: [ :n | DhbVector new: n]) asDhbVector.
	^super initialize: anInteger! !

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Set all accumulators to zero.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	super reset.
	covariance do: [ :each | each atAllPut: 0].! !

!DhbVectorAccumulator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new initialize: anInteger! !

!DhbVectorAccumulator methodsFor: 'transformation' stamp: 'WernerKassens 4/15/2014 09:56'!
accumulate: aVectorOrArray
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	| delta |
	count := count + 1.
	delta := average - aVectorOrArray asDhbVector scaleBy: 1 / count.
	average accumulateNegated: delta.
	^delta! !

!DhbVectorAccumulator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^average! !

!DhbVectorAccumulator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
count
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^count! !

!DhbVectorAccumulator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	average := DhbVector new: anInteger.
	self reset.
	^self! !

!DhbVectorAccumulator methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	super printOn: aStream.
	aStream space.
	count printOn: aStream.
	aStream space.
	average printOn: aStream.! !

!DhbVectorAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	count := 0.
	average atAllPut: 0.! !

Object subclass: #DhbWeightedPoint
	instanceVariableNames: 'xValue yValue weight error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
point: aPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint weight: 1! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
point: aNumber count: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self point: aNumber @ anInteger
		weight: ( anInteger > 0 ifTrue: [ 1 / anInteger]
								ifFalse:[ 1])! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
point: aPoint error: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint error: aNumber! !

!DhbWeightedPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
point: aPoint weight: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^self new initialize: aPoint weight: aNumber! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chi2ComparisonContribution: aWeightedPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^(aWeightedPoint yValue - yValue) squared / ( 1 / aWeightedPoint weight + ( 1 / weight))! !

!DhbWeightedPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chi2Contribution: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^(yValue - ( aFunction value: xValue)) squared * weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
error
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	error isNil
		ifTrue: [ error := 1 / weight sqrt].
	^error! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aPoint error: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	error := aNumber.
	^self initialize: aPoint weight: 1 / aNumber squared! !

!DhbWeightedPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aPoint weight: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	xValue := aPoint x.
	yValue := aPoint y.
	weight := aNumber.
	^self! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
point
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/5/00 "
	^xValue @ yValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
weight
	^weight! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
xValue
	^xValue! !

!DhbWeightedPoint methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
yValue
	^yValue! !

Object subclass: #PointSeries
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!PointSeries class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create a new instance and initialize it"
	^super new initialize! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: anObject
	"Add a point to the receiver"
	self primitiveAdd: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
at: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points at: anInteger! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changedSymbol
	"Answers the symbol of the event sent when the points of the receiver change"
	^#pointsChanged! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectPoints: aBlock
	^points collect: aBlock! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
do: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	self pointsDo: aBlock.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
firstPoint
	"Answers the first point stored in the receiver"
	^self at: 1! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"Create the point collection"
	points := SortedCollection sortBlock: self sortBlock.
	^self! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isEmpty
		"
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^points isEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
notEmpty
	^points notEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointCollection
		" Answer the collection of points.
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^self collectPoints: [ :each | each]! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointsDo: aBlock
	points do: aBlock.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
primitiveAdd: aPoint
	"Private - Add a point to the receiver"
	points add: aPoint.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
primitiveRemove: aPoint
	"Private - Removes a point from the receiver"
	points remove: aPoint.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
remove: anObject
	"Add a point to the receiver"
	self primitiveRemove: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
size
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points size! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sort
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	points := ( points asSortedCollection: self sortBlock) asOrderedCollection.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sortBlock
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^[ :a :b | a x < b x]! !
